# Nengo Example: A Single Neuron
#
# This demo shows you how to construct and manipulate a single leaky
# integrate-and-fire (LIF) neuron. The LIF neuron is a simple, standard
# neuron model, and here it resides inside a neural population, even though
# there is only one neuron.

import nengo
model = nengo.Network()
with model:
    neuron = nengo.Ensemble(n_neurons=1, dimensions=1)
    # Set intercept to 0.5
    neuron.intercepts=nengo.dists.Uniform(-.5, -.5)
    # Set the maximum firing rate of the neuron to 100hz
    neuron.max_rates=nengo.dists.Uniform(100, 100)
    # Sets the neurons firing rate to increase for positive input
    neuron.encoders=[[1]]

    stim = nengo.Node(0)

    # Connect the input signal to the neuron
    nengo.Connection(stim, neuron)


import struct
import collections

import nengo
import nengo.spa
from nengo.spa.module import Module
import numpy as np

from nengo_gui.components.component import Component, Template


class Pointer(Component):
    def __init__(self, viz, config, uid, obj, **kwargs):
        super(Pointer, self).__init__(viz, config, uid)
        self.obj = obj
        self.label = viz.viz.get_label(obj)
        self.data = collections.deque()
        self.override_target = None
        self.target = kwargs.get('args', 'default')
        self.vocab_out = obj.outputs[self.target][1]
        self.vocab_in = obj.inputs[self.target][1]
        self.vocab_out.include_pairs = config.show_pairs

    def add_nengo_objects(self, viz):
        with viz.model:
            output = self.obj.outputs[self.target][0]
            input = self.obj.inputs[self.target][0]
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.vocab_out.dimensions,
                                   size_out=self.vocab_in.dimensions)
            self.conn1 = nengo.Connection(output, self.node, synapse=0.01)
            self.conn2 = nengo.Connection(self.node, input, synapse=0.01)

    def remove_nengo_objects(self, viz):
        viz.model.connections.remove(self.conn1)
        viz.model.connections.remove(self.conn2)
        viz.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        vocab = self.vocab_out
        m = np.dot(vocab.vectors, x)
        matches = [(mm, vocab.keys[i]) for i, mm in enumerate(m) if mm > 0.01]
        if self.config.show_pairs:
            self.vocab_out.include_pairs = True
            m2 = np.dot(vocab.vector_pairs, x)
            matches2 = [(mm, vocab.key_pairs[i]) for i, mm in enumerate(m2)
                        if mm > 0.01]
            matches += matches2
        text = ';'.join(['%0.2f%s' % (sim, key) for (sim, key) in matches])

        msg = '%g %s' % (t, text)
        self.data.append(msg)
        if self.override_target is None:
            return self.vocab_in.parse('0').v
        else:
            v = (self.override_target.v - x) * 3
            if self.vocab_in is not self.vocab_out:
                v = np.dot(self.vocab_out.transform_to(self.vocab_in), v)
            return v


    def update_client(self, client):
        while len(self.data) > 0:
            data = self.data.popleft()
            client.write(data, binary=False)

    def javascript(self):
        info = dict(uid=self.uid, label=self.label)
        json = self.javascript_config(info)
        return 'new Nengo.Pointer(main, sim, %s);' % json

    def message(self, msg):
        if len(msg) == 0:
            self.override_target = None
        else:
            try:
                self.override_target = self.vocab_out.parse(msg)
            except:
                self.override_target = None

    @staticmethod
    def applicable_targets(obj):
        if isinstance(obj, Module):
            return list(obj.outputs.keys())
        else:
            return []


class PointerTemplate(Template):
    cls = Pointer
    config_params = dict(show_pairs=False, **Template.default_params)



# Nengo Network: Ensemble Array
#
# An ensemble array is a group of ensembles that each represent a part of the
# overall signal.
#
# Ensemble arrays are similar to normal ensembles, but expose a slightly
# different interface. Additionally, in an ensemble array, the components of
# the overall signal are not related. As a result, network arrays cannot be
# used to compute nonlinear functions that mix the dimensions they represent.

import nengo
import numpy as np

model = nengo.Network()
with model:
    # Make an input node
    sin = nengo.Node(lambda t: [np.cos(t), np.sin(t)])

    # Make ensembles to connect
    a = nengo.networks.EnsembleArray(n_neurons=100, n_ensembles=2)
    b = nengo.Ensemble(n_neurons=100, dimensions=2)
    c = nengo.networks.EnsembleArray(n_neurons=100, n_ensembles=2)

    # Connect the model elements, just feedforward
    nengo.Connection(sin, a.input)
    nengo.Connection(a.output, b)
    nengo.Connection(b, c.input)


import importlib
import json
import os
import socket
import sys
import time
import threading

import nengo

import nengo_gui
import nengo_gui.server
import nengo_gui.components
import nengo_gui.config
import nengo_gui.monkey
from nengo_gui.components.action import ConfigAction, RemoveGraph


class VizException(Exception):
    pass


class VizSim(object):
    """A single Simulator attached to an html visualization."""

    # list of backends that can only have one Simulator at a time
    singleton_sims = dict(nengo_spinnaker = None)

    def __init__(self, viz):
        self.viz = viz          # the parent Viz organizer
        self.config = viz.config
        self.model = viz.model
        self.building = True    # are we still building the model?
        self.components = []
        self.uids = {}
        self.finished = False   # are we done simulating?
        self.rebuild = False    # should we rebuild the model?
        self._sim = None
        self.changed = False    # has something changed the model, so it
        self.current_error = None
        self.undo_stack = []
        self.redo_stack = []
        self.backend = viz.default_backend
        self.new_code = None
                                #  should be rebuilt?

        for template in self.viz.find_templates():
            self.add_template(template)

        self.net_graph = self.get_net_graph()

        # build and run the model in a separate thread
        t = threading.Thread(target=self.runner)
        t.daemon = True
        t.start()

    @property
    def sim(self):
        return self._sim

    @sim.setter
    def sim(self, value):
        if hasattr(self._sim, 'close'):
            self._sim.close()
        self._sim = value

    def get_net_graph(self):
        for c in self.components:
            if isinstance(c, nengo_gui.components.NetGraph):
                return c
        return None

    def add_template(self, template):
        c = template.create(self)
        self.uids[c.uid] = c
        if isinstance(template, (nengo_gui.components.SimControlTemplate,
                                 nengo_gui.components.NetGraphTemplate,
                                 nengo_gui.components.AceEditorTemplate)):
            self.components[:0] = [c]
        else:
            self.components.append(c)
        return c

    def build(self):
        self.building = True

        # use the lock to make sure only one Simulator is building at a time
        with self.viz.lock:
            for c in self.components:
                c.add_nengo_objects(self.viz)
            # build the simulation
            backend = importlib.import_module(self.backend)
            old_sim = VizSim.singleton_sims.get(self.backend, None)
            if old_sim is not None:
                if old_sim is not self:
                    old_sim.sim = None
                    old_sim.finished = True
            self.sim = backend.Simulator(self.model)
            if self.backend in VizSim.singleton_sims:
                VizSim.singleton_sims[self.backend] = self

            # remove the temporary components added for visualization
            for c in self.components:
                c.remove_nengo_objects(self.viz)
            # TODO: add checks to make sure everything's been removed


        self.building = False

    def runner(self):
        # run the simulation
        while not self.finished:
            if self.sim is None:
                time.sleep(0.01)
            else:
                try:
                    if hasattr(self.sim, 'max_steps'):
                        self.sim.run_steps(self.sim.max_steps)
                    else:
                        self.sim.step()
                except AttributeError:
                    time.sleep(0.01)
                except socket.error:  # if another thread closes the sim
                    pass


            if self.rebuild:
                self.rebuild = False
                self.build()


    def finish(self):
        self.finished = True
        self.viz.remove_sim(self)

    def create_javascript(self):
        fn = json.dumps(self.viz.filename[:-3])
        webpage_title_js = ';document.title = %s;' % fn

        ##Ensure that sim control is first
        temp = self.components[0]
        counter = 0
        for t in self.components:
            if isinstance(t, nengo_gui.components.SimControl):
                self.components[0] = t
                self.components[counter] = temp
                break
            counter += 1

        component_js = '\n'.join([c.javascript() for c in self.components])
        component_js += webpage_title_js
        if not self.viz.allow_file_change:
            component_js += "$('#Open_file_button').addClass('deactivated');"
            pass
        return component_js

    def config_change(self, component, new_cfg, old_cfg):
        act = ConfigAction(self, component=component,
                           new_cfg=new_cfg, old_cfg=old_cfg)
        self.undo_stack.append([act])

    def remove_graph(self, component):
        net_graph = self.get_net_graph()
        act = RemoveGraph(net_graph, component)
        self.undo_stack.append([act])

class Viz(object):
    """The master visualization organizer set up for a particular model."""
    def __init__(
            self, filename=None, model=None, locals=None, cfg=None,
            interactive=True, allow_file_change=True):
        if nengo_gui.monkey.is_executing():
            raise nengo_gui.monkey.StartedVizException()

        self.allow_file_change = allow_file_change

        self.viz_sims = []
        self.cfg = cfg
        self.interactive = interactive
        self.default_backend = 'nengo'

        self.lock = threading.Lock()

        self.config_save_period = 2.0  # minimum time between saves

        if filename is None:
            filename = os.path.join(nengo_gui.__path__[0],
                                    'examples',
                                    'default.py')

        self.load(filename, model, locals, force=True)

    def load(self, filename, model=None, locals=None, force=False,
             reset=False, code=None):
        with self.lock:
            try:
                filename = os.path.relpath(filename)
            except ValueError:
                pass

            if locals is None:
                locals = {}
                locals['nengo_gui'] = nengo_gui
                locals['__file__'] = filename

                if code is not None:
                    self.code = code
                else:
                    try:
                        with open(filename) as f:
                            self.code = f.read()
                    except IOError:
                        self.code = ('import nengo\n\n'
                                    'model = nengo.Network()\n'
                                    'with model:\n'
                                    '    ')

                with nengo_gui.monkey.patch:
                    filedir = os.path.dirname(filename)
                    if filedir not in sys.path:
                        sys.path.insert(0, filedir)
                    else:
                        filedir = None

                    try:
                        exec(self.code, locals)
                    except nengo_gui.monkey.StartedSimulatorException:
                        if self.interactive:
                            line = nengo_gui.monkey.determine_line_number()
                            print('nengo.Simulator() started on line %d. '
                                  'Ignoring all subsequent lines.' % line)
                    except nengo_gui.monkey.StartedVizException:
                        if self.interactive:
                            line = nengo_gui.monkey.determine_line_number()
                            print('nengo_gui.Viz() started on line %d. '
                                  'Ignoring all subsequent lines.' % line)
                    except Exception as e:
                        if not force:
                            raise
                        else:
                            print("Error loading file: %s" % e)
                    finally:
                        if filedir is not None:
                            sys.path.remove(filedir)

            self.orig_locals = dict(locals)

            if model is None:
                if 'model' not in locals:
                    if force:
                        locals['model'] = nengo.Network()
                    else:
                        raise VizException('No object called "model" in the code')
                model = locals['model']
                if not isinstance(model, nengo.Network):
                    if force:
                        locals['model'] = nengo.Network()
                        model = locals['model']
                    else:
                        raise VizException('The "model" must be a nengo.Network')



            self.model = model
            self.locals = dict(locals)

            self.filename = filename
            self.name_finder = nengo_gui.NameFinder(locals, model)
            self.default_labels = self.name_finder.known_name

            if reset:
                if os.path.isfile(self.config_name()) :
                    os.remove(self.config_name())

            self.config = self.load_config()
            self.config_save_needed = False
            self.config_save_time = None   # time of last config file save


            self.uid_prefix_counter = {}

    def find_templates(self):
        for k, v in self.locals.items():
            if isinstance(v, nengo_gui.components.component.Template):
                yield v

    def generate_uid(self, obj, prefix):
        index = self.uid_prefix_counter.get(prefix, 0)
        uid = '%s%d' % (prefix, index)
        while uid in self.locals:
            index += 1
            uid = '%s%d' % (prefix, index)
        self.uid_prefix_counter[prefix] = index + 1

        self.locals[uid] = obj
        self.default_labels[obj] = uid

    def remove_uid(self, uid):
        if uid in self.locals:
            obj = self.locals[uid]
            del self.locals[uid]
            del self.default_labels[obj]

    def config_name(self):
        if self.cfg is None:
            return self.filename + '.cfg'
        else:
            return self.cfg

    def load_config(self):
        config = nengo_gui.config.Config()
        self.locals['nengo_gui'] = nengo_gui
        self.locals['_viz_config'] = config
        fname = self.config_name()
        if os.path.exists(fname):
            with open(fname) as f:
                config_code = f.readlines()
            for line in config_code:
                try:
                    exec(line, self.locals)
                except Exception as e:
                    if self.interactive:
                        print('error parsing config', line, e)

        # make sure a SimControl and a NetGraph exist
        if '_viz_sim_control' not in self.locals:
            template = nengo_gui.components.SimControlTemplate()
            self.locals['_viz_sim_control'] = template
        if '_viz_net_graph' not in self.locals:
            template = nengo_gui.components.NetGraphTemplate()
            self.locals['_viz_net_graph'] = template
        if '_viz_ace_editor' not in self.locals:
            template = nengo_gui.components.AceEditorTemplate()
            self.locals['_viz_ace_editor'] = template

        if config[self.model].pos is None:
            config[self.model].pos = (0, 0)
        if config[self.model].size is None:
            config[self.model].size = (1.0, 1.0)

        for k, v in self.locals.items():
            if isinstance(v, nengo_gui.components.component.Template):
                self.default_labels[v] = k
        return config

    def save_config(self, lazy=False, force=False):
        if not force and not self.config_save_needed:
            return

        now_time = time.time()
        if not force and lazy and self.config_save_time is not None:
            if (now_time - self.config_save_time) < self.config_save_period:
                return

        with self.lock:
            self.config_save_time = now_time
            self.config_save_needed = False
            try:
                with open(self.config_name(), 'w') as f:
                    f.write(self.config.dumps(uids=self.default_labels))
            except IOError:
                print("Could not save %s; permission denied" %
                      self.config_name())

    def modified_config(self):
        self.config_save_needed = True

    def get_label(self, obj, default_labels=None):
        if default_labels is None:
            default_labels = self.default_labels
        label = obj.label
        if label is None:
            label = default_labels.get(obj, None)
            if label is None:
                print('ERROR finding label: %s' % obj)
            else:
                if '.' in label:
                    label = label.rsplit('.', 1)[1]
        if label is None:
            label = repr(obj)
        return label

    def get_uid(self, obj, default_labels=None):
        if default_labels is None:
            default_labels = self.default_labels
        uid = default_labels.get(obj, None)
        if uid is None:
            uid = repr(obj)
        return uid

    def start(self, port=8080, browser=True, password=None):
        """Start the web server"""
        print("Starting nengo_gui server at http://localhost:%d" % port)
        if password is not None:
            nengo_gui.server.Server.add_user('', password)
            addr = ''
        else:
            addr = 'localhost'
        nengo_gui.server.Server.start(self, port=port, browser=browser, addr=addr)

    def prepare_server(self, viz, port=8080, browser=True):
        return nengo_gui.server.Server.prepare_server(
            self, port=port, browser=browser)

    def begin_lifecycle(self, server):
        nengo_gui.server.Server.begin_lifecycle(
            server, interactive=self.interactive)

    def create_sim(self):
        """Create a new Simulator with this configuration"""
        viz_sim = VizSim(self)
        self.viz_sims.append(viz_sim)
        return viz_sim

    def remove_sim(self, viz_sim):
        self.viz_sims.remove(viz_sim)

    def count_sims(self):
        return len(self.viz_sims)



"""One-off functions that create Javascript calls"""

import json

import numpy as np

import nengo
from nengo.utils.ensemble import response_curves, tuning_curves


def infomodal(ng, uid, **args):
    obj = ng.uids[uid]
    if isinstance(obj, nengo.Ensemble):
        return ensemble_infomodal(ng, uid=uid, **args)
    elif isinstance(obj, nengo.Node):
        return node_infomodal(ng, uid=uid, **args)
    elif isinstance(obj, nengo.Network):
        return net_infomodal(ng, uid=uid, **args)
    else:
        raise NotImplementedError()

def add_modal_title_js(title_text):
    return 'Nengo.modal.title("%s");' % (title_text)

def add_modal_footer_js(footer_text):
    return 'Nengo.modal.footer("%s");' % (footer_text)

def show_modal_js():
    return 'Nengo.modal.show();'

class PlotInfo(object):
    def __init__(self, title, plot="none"):
        self.title = title
        self.plot = plot
        self.warnings = []
        self.x = None
        self.y = None

    def to_dict(self):
        x, y = self.x, self.y
        if self.plot == "multiline":
            assert self.x.shape[0] == self.y.shape[1]
            x = self.x.tolist()
            y = [yy.tolist() for yy in self.y]
        return {
            'plot': self.plot,
            'title': self.title,
            'warnings': self.warnings,
            'x': x,
            'y': y,
        }

def ensemble_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    ens = ng.uids[uid]

    params = [(attr, str(getattr(ens, attr))) for attr in (
        'n_neurons', 'dimensions', 'radius', 'encoders', 'intercepts',
        'max_rates', 'eval_points', 'n_eval_points', 'neuron_type',
        'noise', 'seed') if getattr(ens, attr) is not None]

    if ng.viz.sim is None:
        plots = ("Simulation not yet running. "
                 "Start a simulation to see plots.")
    else:
        plots = []
        rc = PlotInfo("Response curves", plot="multiline")
        rc.x, rc.y = response_curves(ens, ng.viz.sim)
        rc.y = rc.y.T
        if ens.n_neurons > 200:
            rc.warnings.append("Only showing the first 200 neurons.")
            rc.y = rc.y[:200]
        plots.append(rc.to_dict())

        tc = PlotInfo("Tuning curves")
        if ens.dimensions == 1:
            tc.plot = "multiline"
            tc.x, tc.y = tuning_curves(ens, ng.viz.sim)
            tc.y = tc.y.T
            if ens.n_neurons > 200:
                tc.warnings.append("Only showing the first 200 neurons.")
                tc.y = tc.y[:200]
        else:
            tc.warnings.append("Tuning curves only shown for "
                               "one-dimensional ensembles.")
        plots.append(tc.to_dict())

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = ['Nengo.modal.title("Details for \'%s\'");' % ng.viz.viz.get_label(ens)]
    js.append('Nengo.modal.footer("close");')
    js.append('Nengo.modal.ensemble_body("%s", %s, %s, %s);' % (
        uid, json.dumps(params), json.dumps(plots), json.dumps(conninfo)))
    js.append('Nengo.modal.show();')
    return '\n'.join(js)

def node_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    node = ng.uids[uid]

    params = [(attr, str(getattr(node, attr))) for attr in (
        'output', 'size_in', 'size_out') if getattr(node, attr) is not None]

    f_out = PlotInfo("Node output")
    if node.size_in > 0:
        f_out.warnings.append("Node output only shown when 'size_in' is 0.")
    else:
        f_out.plot = "multiline"
        if callable(node.output):
            dt = 0.001
            f_out.x = np.arange(dt, 1.0, dt)
            f_out.y = np.asarray([node.output(x) for x in f_out.x])
        else:
            # Don't bother with all the copies if it's static
            f_out.x = np.asarray([0, 1.0])
            f_out.y = np.hstack((node.output, node.output))
        if f_out.y.ndim == 1:
            f_out.y = f_out.y[:, np.newaxis]
        f_out.y = f_out.y.T

    plots = [f_out.to_dict()]

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = [add_modal_title_js("Details for \'%s\'" % (
        ng.viz.viz.get_label(node)))]
    js.append(add_modal_footer_js('close'))
    js.append('Nengo.modal.node_body("%s", %s, %s, %s);' % (
        uid, json.dumps(params), json.dumps(plots), json.dumps(conninfo)))
    js.append(show_modal_js())
    return '\n'.join(js)


def conn_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    conninfo = {}

    conninfo["obj_type"] = {}
    conninfo["func"] = {}
    conninfo["fan"] = {}

    def get_conn_func_str(conn_uid):
        return (str(ng.uids[conn_uid].function)
                if ng.uids[conn_uid].function is not None
                else "Identity function")

    def get_obj_info(nengo_obj):
        if isinstance(nengo_obj, nengo.Node):
            if nengo_obj.output is None:
                return "passthrough", "> 0"
            else:
                return "node", "0"
        elif isinstance(nengo_obj, nengo.Ensemble):
            return "ens", str(nengo_obj.n_neurons)
        elif isinstance(nengo_obj, nengo.Network):
            return "net", str(sum(map(lambda e: e.n_neurons,
                                      nengo_obj.all_ensembles)))
        else:
            return "err", "0"

    for conn_uid in conn_in_uids:
        conninfo["obj_type"][conn_uid], conninfo["fan"][conn_uid] = \
            get_obj_info(ng.uids[conn_uid].pre_obj)
        conninfo["func"][conn_uid] = get_conn_func_str(conn_uid)

    for conn_uid in conn_out_uids:
        conninfo["obj_type"][conn_uid], conninfo["fan"][conn_uid] = \
            get_obj_info(ng.uids[conn_uid].post_obj)
        conninfo["func"][conn_uid] = get_conn_func_str(conn_uid)

    return conninfo


def net_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    net = ng.uids[uid]

    stats = []
    stats.append({
        'title': "In this network",
        'stats': [
            ('Number of ensembles', len(net.ensembles)),
            ('Total number of neurons',
             sum(e.n_neurons for e in net.ensembles)),
            ('Number of nodes', len(net.nodes)),
            ('Number of connections', len(net.connections)),
            ('Number of subnetworks', len(net.networks)),
        ]})
    stats.append({
        'title': "In this network and subnetworks",
        'stats': [
            ('Number of ensembles', len(net.all_ensembles)),
            ('Total number of neurons',
             sum(e.n_neurons for e in net.all_ensembles)),
            ('Number of nodes', len(net.all_nodes)),
            ('Number of connections', len(net.all_connections)),
            ('Number of subnetworks', len(net.all_networks)),
        ]})

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = [add_modal_title_js("Details for \'%s\'") % (
        ng.viz.viz.get_label(net))]
    js.append(add_modal_footer_js('close'))
    js.append('Nengo.modal.net_body("%s", %s, %s);' % (
        uid, json.dumps(stats), json.dumps(conninfo)))
    js.append(show_modal_js())
    return '\n'.join(js)



import argparse

import nengo_gui

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-p', '--password', dest='password', metavar='PASS',
        help='password for remote access')
    parser.add_argument(
        '-P', '--port', dest='port', metavar='PORT',
        default=8080, type=int, help='port to run server on')
    parser.add_argument(
        'filename', nargs='?', type=str, help='initial file to load')
    parser.add_argument('--debug', action='store_true',
        help='turn on debug logging')
    parser.add_argument(
        '-b', '--backend', metavar='BACKEND',
        default='nengo', type=str, help='default backend to use')
    args = parser.parse_args()

    if args.debug:
        import logging
        logging.basicConfig(level=logging.DEBUG)

    viz = nengo_gui.Viz(filename=args.filename)
    viz.default_backend = args.backend
    viz.start(port=args.port, password=args.password)

if __name__ == '__main__':
    main()



import json

class Component(object):
    def __init__(self, viz, config, uid):
        self.config = config
        self.uid = uid

    def update_client(self, client):
        pass
    def message(self, msg):
        print('unhandled message', msg)

    def finish(self):
        pass

    def add_nengo_objects(self, viz):
        pass

    def remove_nengo_objects(self, viz):
        pass

    def javascript_config(self, cfg):
        for attr in self.config._clsparams.params:
            cfg[attr] = getattr(self.config, attr)
        return json.dumps(cfg)


class Template(object):
    default_params = dict(x=0, y=0, width=100, height=100, label_visible=True)

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
    def create(self, vizsim):
        uid = vizsim.viz.get_uid(self)
        c = self.cls(vizsim, vizsim.viz.config[self], uid,
                     *self.args, **self.kwargs)
        c.template = self
        return c
    def code_python(self, uids):
        args = [uids[x] for x in self.args]
        name = self.__class__.__name__
        return 'nengo_gui.components.%s(%s)' % (name, ','.join(args))



import atexit
import socket
import threading
import time
import urllib2
import uuid
import warnings
import weakref

from IPython import get_ipython
from IPython.display import display, HTML

import nengo_gui


class ConfigReuseWarning(UserWarning):
    pass


class IPythonViz(object):
    servers = weakref.WeakValueDictionary()
    threads = weakref.WeakValueDictionary()
    configs = set()

    host = 'localhost'

    def __init__(self, model, cfg=None, height=600):
        nengo_gui.server.Server.log_file = None

        self.height = height

        self._started = False

        if cfg is None:
            cfg = get_ipython().mktempfile()

        self._server_thread, server = self.start_server(cfg, model)
        self.port = server.server_port

        self.url = self.get_url(self.host, self.port)


    @staticmethod
    def get_url(host, port, action=None):
        url = 'http://{host}:{port}'.format(host=host, port=port)
        if action is not None:
            url += '/' + action
        return url

    @classmethod
    def start_server(cls, cfg, model):
        # Make sure only one server is writing the same config.
        server_thread = cls.threads.get(cfg, None)
        server = cls.servers.get(cfg, None)
        existent = server_thread is not None and server is not None
        if existent and server_thread.is_alive():
            warnings.warn(ConfigReuseWarning(
                "Reusing config. Only the most recent visualization will "
                "update the config."))
            server.viz.save_config(force=True)
            server.viz.cfg = get_ipython().mktempfile()
            cls.servers[server.viz.cfg] = server
            cls.threads[server.viz.cfg] = server_thread

        name = model.label if model.label is not None else ''
        viz = nengo_gui.Viz(
            name, cfg=cfg, model=model, locals=get_ipython().user_ns,
            interactive=False, allow_file_change=False)
        server = viz.prepare_server(viz, port=0, browser=False)
        server_thread = threading.Thread(
            target=viz.begin_lifecycle,
            kwargs={'server': server})
        server_thread.start()
        cls.servers[cfg] = server
        cls.threads[cfg] = server_thread
        cls.configs.add(cfg)
        return server_thread, server

    def wait_for_startup(self):
        while self._server_thread.is_alive() and not self._started:
            try:
                s = socket.create_connection((self.host, self.port), 0.1)
                self._started = True
            except:
                pass
            else:
                s.close()

    @classmethod
    def shutdown_all(cls, timeout=None):
        def get_timeout(start=time.clock()):
            if timeout is None:
                return None
            else:
                return max(0.001, timeout - (time.clock() - start))

        for cfg in cls.configs:
            server = cls.servers.get(cfg, None)
            server_thread = cls.threads.get(cfg, None)
            if server_thread is not None and server_thread.is_alive():
                if server is not None:
                    urllib2.urlopen(
                        cls.get_url(cls.host, server.server_port, 'shutdown'),
                        timeout=get_timeout()).read()
                server_thread.join(get_timeout())

    def _ipython_display_(self):
        self.wait_for_startup()
        if self._server_thread.is_alive():
            display(HTML('''
                <div id="{id}">
                    <iframe
                        src="{url}"
                        width="100%"
                        height="{height}"
                        frameborder="0"
                        class="cell"
                        style="border: 1px solid #eee;"
                        allowfullscreen></iframe>
                </div>
            '''.format(url=self.url, id=uuid.uuid4(), height=self.height)))
        else:
            print "Server is not alive."


atexit.register(IPythonViz.shutdown_all, timeout=5)



import struct
import collections

import nengo
import numpy as np

from nengo_gui.components.component import Component, Template


class Value(Component):
    def __init__(self, viz, config, uid, obj):
        super(Value, self).__init__(viz, config, uid)
        self.obj = obj
        self.label = viz.viz.get_label(obj)
        self.data = collections.deque()
        self.n_lines = int(obj.size_out)
        self.struct = struct.Struct('<%df' % (1 + self.n_lines))

    def add_nengo_objects(self, viz):
        with viz.model:
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.obj.size_out)
            self.conn = nengo.Connection(self.obj, self.node, synapse=None)

    def remove_nengo_objects(self, viz):
        viz.model.connections.remove(self.conn)
        viz.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        self.data.append(self.struct.pack(t, *x))

    def update_client(self, client):
        while len(self.data) > 0:
            item = self.data.popleft()
            client.write(item, binary=True)

    def javascript(self):
        info = dict(uid=self.uid, label=self.label,
                    n_lines=self.n_lines, synapse=0.01)
        json = self.javascript_config(info)
        return 'new Nengo.Value(main, sim, %s);' % json

class ValueTemplate(Template):
    cls = Value
    config_params = dict(max_value=1, min_value=-1, **Template.default_params)



# Nengo Example: Multiplication

# This example will show you how to multiply two values. The model
# architecture can be thought of as a combination of the combining demo and
# the squaring demo. Essentially, we project both inputs independently into a
# 2D space, and then decode a nonlinear transformation of that space (the
# product of the first and second vector elements).

import nengo
model = nengo.Network()
with model:
    # Create 4 ensembles of leaky integrate-and-fire neurons
    a = nengo.Ensemble(n_neurons=100, dimensions=1, radius=1)
    b = nengo.Ensemble(n_neurons=100, dimensions=1, radius=1)

    # Radius on this ensemble is ~sqrt(2)
    combined = nengo.Ensemble(n_neurons=200, dimensions=2, radius=1.5)

    prod = nengo.Ensemble(n_neurons=100, dimensions=1, radius=1)

    
    # These next two lines make all of the encoders in the Combined population
    # point at the corners of the cube. This improves the quality of the
    # computation.
    combined.encoders = nengo.dists.Choice([[1,1],[-1,1],[1,-1],[-1,-1]])
    
    stim_a = nengo.Node([0])
    stim_b = nengo.Node([0])

    # Connect the input nodes to the appropriate ensembles
    nengo.Connection(stim_a, a)
    nengo.Connection(stim_b, b)

    # Connect input ensembles A and B to the 2D combined ensemble
    nengo.Connection(a, combined[0])
    nengo.Connection(b, combined[1])

    # Define a function that computes the multiplication of two inputs
    def product(x):
        return x[0] * x[1]

    # Connect the combined ensemble to the output ensemble
    nengo.Connection(combined, prod, function=product)


import nengo
import nengo.spa as spa

D = 16

model = spa.SPA(seed=1)
with model:
    model.a = spa.Buffer(dimensions=D)
    model.b = spa.Buffer(dimensions=D)
    model.c = spa.Buffer(dimensions=D)
    model.cortical = spa.Cortical(spa.Actions(
        'c = a+b',
        ))

    model.input = spa.Input(
        a = 'A',
        b = (lambda t: 'C*~A' if (t%0.1 < 0.05) else 'D*~A'),
        )

if __name__ == '__main__':
    import nengo_gui
    nengo_gui.Viz(__file__).start()



from __future__ import division

import nengo
import numpy as np
import struct

from nengo_gui.components.component import Component, Template


class Voltage(Component):
    def __init__(self, viz, config, uid, obj, n_neurons=5):
        super(Voltage, self).__init__(viz, config, uid)
        self.viz = viz
        self.obj = obj.neurons
        self.data = []
        self.label = viz.viz.get_label(obj)
        self.max_neurons = int(self.obj.size_out)
        self.n_neurons = min(n_neurons, self.max_neurons)
        self.struct = struct.Struct('<%df' % (1 + self.n_neurons))

    def add_nengo_objects(self, viz):
        with viz.model:
            self.probe = nengo.Probe(self.obj[:self.n_neurons], 'voltage')

    def remove_nengo_objects(self, viz):
        viz.model.probes.remove(self.probe)

    def format_data(self, t, x):
        data = self.struct.pack(t, *x[:self.n_neurons])
        self.data.append(data)

    def update_client(self, client):
        sim = self.viz.sim
        if sim is None:
            return

        # TODO: this is hack to delete probe data in Nengo 2.0.1, since we
        # can't limit the size of probes. Fix this up with Nengo 2.1.
        data = sim.data.raw[self.probe][:]
        del sim.data.raw[self.probe][:]  # clear the data
        trange = sim.trange()[-len(data):]

        for t, datum in zip(trange, data):
            datum = (datum + np.arange(self.n_neurons))
            packet = self.struct.pack(t, *datum)
            client.write(packet, binary=True)

    def javascript(self):
        info = dict(uid=self.uid, label=self.label,
                    n_lines=self.n_neurons, synapse=0)
        json = self.javascript_config(info)
        return 'new Nengo.Value(main, sim, %s);' % json


class VoltageTemplate(Template):
    cls = Voltage
    config_params = dict(
        max_value=5.0, min_value=0.0, **Template.default_params)



import time
import os
import traceback
import collections

import nengo
import json

from nengo_gui.components.component import Component, Template
from nengo_gui.disposable_js import infomodal
import nengo_gui.layout

from .action import create_action


class NetGraph(Component):
    configs = {}

    def __init__(self, viz, config, uid):
        super(NetGraph, self).__init__(viz, config, uid)
        self.viz = viz
        self.layout = nengo_gui.layout.Layout(self.viz.model)
        self.config = viz.config
        self.to_be_expanded = collections.deque([self.viz.model])
        self.to_be_sent = collections.deque()
        self.uids = {}
        self.parents = {}
        self.networks_to_search = [self.viz.model]
        self.initialized_pan_and_zoom = False
        try:
            self.last_modify_time = os.path.getmtime(self.viz.viz.filename)
        except OSError:
            self.last_modify_time = None
        self.last_reload_check = time.time()

    def check_for_reload(self):
        try:
            t = os.path.getmtime(self.viz.viz.filename)
        except OSError:
            t = None

        if t is not None:
            if self.last_modify_time is None or self.last_modify_time < t:
                self.reload()
                self.last_modify_time = t

        new_code = self.viz.new_code
        self.viz.new_code = None
        if new_code is not None:
            self.reload(code=new_code)

    def reload(self, code=None):
        with self.viz.viz.lock:
            self._reload(code=code)

    def _reload(self, code=None):
        current_error = None
        locals = {}
        if code is None:
            with open(self.viz.viz.filename) as f:
                code = f.read()

        try:
            with nengo_gui.monkey.patch:
                exec(code, locals)
        except:
            line = nengo_gui.monkey.determine_line_number()
            current_error = dict(trace=traceback.format_exc(), line=line)
            traceback.print_exc()
            self.viz.current_error = current_error
            return
        try:
            model = locals['model']
        except:
            if current_error is None:
                line = nengo_gui.monkey.determine_line_number()
                current_error = dict(trace=traceback.format_exc(), line=line)
                traceback.print_exc()
            self.viz.current_error = current_error
            return
        self.viz.current_error = current_error

        locals['nengo_gui'] = nengo_gui
        name_finder = nengo_gui.NameFinder(locals, model)

        self.networks_to_search = [model]
        self.parents = {}

        removed_uids = {}

        # for each item in the old model, find the matching new item
        # for Nodes, Ensembles, and Networks, this means to find the item
        # with the same uid.  For Connections, we don't really have a uid,
        # so we use the uids of the pre and post objects.
        for uid, old_item in nengo.utils.compat.iteritems(dict(self.uids)):
            try:
                new_item = eval(uid, locals)
            except:
                new_item = None

            # check to make sure the new item's uid is the same as the
            # old item.  This is to catch situations where an old uid
            # happens to still refer to something in the new model, but that's
            # not the normal uid for that item.  For example, the uid
            # "ensembles[0]" might still refer to something even after that
            # ensemble is removed.
            new_uid = self.viz.viz.get_uid(new_item,
                        default_labels=name_finder.known_name)
            if new_uid != uid:
                new_item = None

            # find reasons to delete the object.  Any deleted object will
            # be recreated, so try to keep this to a minimum
            keep_object = True
            if new_item is None:
                keep_object = False
            elif not isinstance(new_item, old_item.__class__):
                # don't allow changing classes
                keep_object = False
            elif isinstance(new_item, nengo.Node):
                # check if a Node has become a passthrough Node
                if old_item.output is None and new_item.output is not None:
                    keep_object = False
                elif old_item.output is not None and new_item.output is None:
                    keep_object = False

            if not keep_object:
                self.to_be_sent.append(dict(
                    type='remove', uid=uid))
                del self.uids[uid]
                removed_uids[old_item] = uid
            else:
                if isinstance(old_item, (nengo.Node,
                                         nengo.Ensemble,
                                         nengo.Network)):
                    old_label = self.viz.viz.get_label(old_item)
                    new_label = self.viz.viz.get_label(
                        new_item, default_labels=name_finder.known_name)

                    if old_label != new_label:
                        self.to_be_sent.append(dict(
                            type='rename', uid=uid, name=new_label))
                    if isinstance(old_item, nengo.Network):
                        if self.viz.viz.config[old_item].expanded:
                            self.to_be_expanded.append(new_item)

                elif isinstance(old_item, nengo.Connection):
                    old_pre = old_item.pre_obj
                    old_post = old_item.post_obj
                    new_pre = new_item.pre_obj
                    new_post = new_item.post_obj
                    if isinstance(old_pre, nengo.ensemble.Neurons):
                        old_pre = old_pre.ensemble
                    if isinstance(old_post, nengo.ensemble.Neurons):
                        old_post = old_post.ensemble
                    if isinstance(new_pre, nengo.ensemble.Neurons):
                        new_pre = new_pre.ensemble
                    if isinstance(new_post, nengo.ensemble.Neurons):
                        new_post = new_post.ensemble

                    old_pre = self.viz.viz.get_uid(old_pre)
                    old_post = self.viz.viz.get_uid(old_post)
                    new_pre = self.viz.viz.get_uid(
                        new_pre, default_labels=name_finder.known_name)
                    new_post = self.viz.viz.get_uid(
                        new_post, default_labels=name_finder.known_name)

                    if new_pre != old_pre or new_post != old_post:
                        # if the connection has changed, tell javascript
                        pres = self.get_parents(
                            new_pre,
                            default_labels=name_finder.known_name)[:-1]
                        posts = self.get_parents(
                            new_post,
                            default_labels=name_finder.known_name)[:-1]
                        self.to_be_sent.append(dict(
                            type='reconnect', uid=uid,
                            pres=pres, posts=posts))

                self.uids[uid] = new_item

        self.to_be_expanded.append(model)

        self.viz.model = model
        self.viz.viz.model = model
        self.viz.viz.locals = locals
        self.viz.viz.name_finder = name_finder
        self.viz.viz.default_labels = name_finder.known_name
        self.viz.viz.config = self.viz.viz.load_config()
        self.viz.config = self.viz.viz.config
        self.config = self.viz.viz.config
        self.viz.viz.uid_prefix_counter = {}
        self.layout = nengo_gui.layout.Layout(model)
        self.viz.viz.code = code


        removed_items = list(removed_uids.keys())
        for c in self.viz.components:
            for item in c.template.args:
                if item in removed_items:
                    self.to_be_sent.append(dict(type='delete_graph',
                                                uid=c.uid))
                    break

        components = []
        for c in self.viz.components[:3]:
            components.append(c)
            locals[c.uid] = c.template
        self.viz.components = components
        for template in self.viz.viz.find_templates():
            if not isinstance(template,
                              (nengo_gui.components.SimControlTemplate,
                               nengo_gui.components.NetGraphTemplate,
                               nengo_gui.components.AceEditorTemplate)):
                try:
                    self.viz.add_template(template)
                except:
                    print('failed to recreate plot for %s' % template)

        self.viz.changed = True

    def get_parents(self, uid, default_labels=None):
        while uid not in self.parents:
            net = self.networks_to_search.pop(0)
            net_uid = self.viz.viz.get_uid(net, default_labels=default_labels)
            for n in net.nodes:
                n_uid = self.viz.viz.get_uid(n, default_labels=default_labels)
                self.parents[n_uid] = net_uid
            for e in net.ensembles:
                e_uid = self.viz.viz.get_uid(e, default_labels=default_labels)
                self.parents[e_uid] = net_uid
            for n in net.networks:
                n_uid = self.viz.viz.get_uid(n, default_labels=default_labels)
                self.parents[n_uid] = net_uid
                self.networks_to_search.append(n)
        parents = [uid]
        while parents[-1] in self.parents:
            parents.append(self.parents[parents[-1]])
        return parents

    def modified_config(self):
        self.viz.viz.modified_config()

    def update_client(self, client):
        now = time.time()
        if now > self.last_reload_check + 0.5:
            self.check_for_reload()
            self.last_reload_check = now

        if not self.initialized_pan_and_zoom:
            self.send_pan_and_zoom(client)
            self.initialized_pan_and_zoom = True

        while len(self.to_be_sent) > 0:
            info = self.to_be_sent.popleft()
            client.write(json.dumps(info))

        if len(self.to_be_expanded) > 0:
            self.viz.viz.lock.acquire()
            network = self.to_be_expanded.popleft()
            self.expand_network(network, client)
            self.viz.viz.lock.release()

    def javascript(self):
        return 'new Nengo.NetGraph(main, {uid:"%s"});' % self.uid

    def message(self, msg):
        try:
            info = json.loads(msg)
        except ValueError:
            print('invalid message', repr(msg))
            return
        action = info.get('act', None)
        undo = info.get('undo', None)
        if action is not None:
            del info['act']
            if action in ('auto_expand', 'auto_collapse'):
                getattr(self, 'act_' + action[5:])(**info)
            elif action in ('pan', 'zoom', 'create_modal'):
                # These should not use the undo stack
                getattr(self, 'act_' + action)(**info)
            else:
                act = create_action(action, self, **info)
                self.viz.undo_stack.append([act])
                del self.viz.redo_stack[:]
        elif undo is not None:
            if undo == '1':
                self.undo()
            else:
                self.redo()
        else:
            print('received message', msg)

    def undo(self):
        if self.viz.undo_stack:
            action = self.viz.undo_stack.pop()
            re = []
            for act in action:
                act.undo()
                re.insert(0, act)
            self.viz.redo_stack.append(re)

    def redo(self):
        if self.viz.redo_stack:
            action = self.viz.redo_stack.pop()
            un = []
            for act in action:
                act.apply()
                un.insert(0, act)
            self.viz.undo_stack.append(un)

    def act_expand(self, uid):
        net = self.uids[uid]
        self.to_be_expanded.append(net)
        self.config[net].expanded = True
        self.modified_config()

    def act_collapse(self, uid):
        net = self.uids[uid]
        self.config[net].expanded = False
        self.remove_uids(net)
        self.modified_config()

    def remove_uids(self, net):
        for items in [net.ensembles, net.networks, net.nodes, net.connections]:
            for item in items:
                uid = self.viz.viz.get_uid(item)
                if uid in self.uids:
                    del self.uids[uid]
        for n in net.networks:
            self.remove_uids(n)

    def act_pan(self, x, y):
        self.config[self.viz.model].pos = x, y
        self.modified_config()

    def act_zoom(self, scale, x, y):
        self.config[self.viz.model].size = scale, scale
        self.config[self.viz.model].pos = x, y
        self.modified_config()

    def act_create_modal(self, uid, **info):
        js = infomodal(self, uid, **info)
        self.to_be_sent.append(dict(type='js', code=js))

    def expand_network(self, network, client):
        if not self.config[network].has_layout:
            pos = self.layout.make_layout(network)
            for obj, layout in pos.items():
                self.config[obj].pos = layout['y'], layout['x']
                self.config[obj].size = layout['h'] / 2, layout['w'] / 2
            self.config[network].has_layout = True

        if network is self.viz.model:
            parent = None
        else:
            parent = self.viz.viz.get_uid(network)
        for ens in network.ensembles:
            self.create_object(client, ens, type='ens', parent=parent)
        for node in network.nodes:
            self.create_object(client, node, type='node', parent=parent)
        for net in network.networks:
            self.create_object(client, net, type='net', parent=parent)
        for conn in network.connections:
            self.create_connection(client, conn, parent=parent)
        self.config[network].expanded = True

    def create_object(self, client, obj, type, parent):
        uid = self.viz.viz.get_uid(obj)
        if uid in self.uids:
            return

        pos = self.config[obj].pos
        if pos is None:
            import random
            pos = random.uniform(0, 1), random.uniform(0, 1)
            self.config[obj].pos = pos
        size = self.config[obj].size
        if size is None:
            size = (0.1, 0.1)
            self.config[obj].size = size
        label = self.viz.viz.get_label(obj)
        self.uids[uid] = obj
        info = dict(uid=uid, label=label, pos=pos, type=type, size=size,
                    parent=parent)
        if type == 'net':
            info['expanded'] = self.config[obj].expanded
        if type == 'node' and obj.output is None:
            info['passthrough'] = True
        if type == 'ens' or type == 'node':
            info['dimensions'] = int(obj.size_out)

        info['sp_targets'] = (
            nengo_gui.components.pointer.Pointer.applicable_targets(obj))

        client.write(json.dumps(info))

    def send_pan_and_zoom(self, client):
        pan = self.config[self.viz.model].pos
        if pan is None:
            pan = 0, 0
        zoom = self.config[self.viz.model].size
        if zoom is None:
            zoom = 1.0
        else:
            zoom = zoom[0]
        client.write(json.dumps(dict(type='pan', pan=pan)))
        client.write(json.dumps(dict(type='zoom', zoom=zoom)))

    def create_connection(self, client, conn, parent):
        uid = self.viz.viz.get_uid(conn)
        if uid in self.uids:
            return
        pre = conn.pre_obj
        if isinstance(pre, nengo.ensemble.Neurons):
            pre = pre.ensemble
        post = conn.post_obj
        if isinstance(post, nengo.ensemble.Neurons):
            post = post.ensemble
        pre = self.viz.viz.get_uid(pre)
        post = self.viz.viz.get_uid(post)
        self.uids[uid] = conn
        pres = self.get_parents(pre)[:-1]
        posts = self.get_parents(post)[:-1]
        info = dict(uid=uid, pre=pres, post=posts, type='conn', parent=parent)
        client.write(json.dumps(info))


class NetGraphTemplate(Template):
    cls = NetGraph
    config_params = dict()



"""
Simple Web Interface
Author: Terry Stewart terry.stewart@gmail.com http://terrystewart.ca

This software is released under the GNU General Public License.
See http://www.gnu.org/copyleft/gpl.html for more details.


Simple Web Interface is a fast way to create Python programs that serve
  dynamic web pages, without a heavyweight infrastructure.  Everything is
  in this one file.  You simply define a class, and the methods in that
  class will be called to create the various web pages.

An HTTP request for http://server/page?a=val1&b=val2 will result in a call to
    def swi_page(self,a,b)
  and 'a' will get the value 'val1' and 'b' will be 'val2'.  You can also
  define defaults for these arguments, and it will use those if they
  are unspecified.  This method will also be called for a request like this:
    http://server/page/val1/val2
  There is no limitation on the number of arguments.  The system supports
  both GET and POST (and multipart POST, so file uploading will work).

The system also supports a cookie-based login system.  You can define users
  with swi.addUser(name,password).  You can then check for logins by adding
  this to the beginning of your methods:
    if self.user==None:
      return self.default_login_form()
  You can log out by causing a call to self.log_out().

Websockets are also supported via functions that begin with ws_:
    # ws://server/data
    def ws_data(self, client):
        while True:
            msg = client.read()
            if msg is not None:
                client.write('received: ' + msg)
"""

import base64
try:
    import BaseHTTPServer
except ImportError:
    import http.server as BaseHTTPServer
import errno
import hashlib
try:
    import mimetools
except ImportError:
    import email as mimetools
import mimetypes
try:
    import multifile
except ImportError:
    import email as multifile
import os
import random
import re
import select
import signal
import socket
try:
    import SocketServer
except ImportError:
    import socketserver as SocketServer
import string
import struct
try:
    import StringIO
except ImportError:
    import io as StringIO
import sys
import threading
import time
import traceback
import weakref
import webbrowser


class SocketClosedError(IOError):
    pass


# Does not really indicate an error state, thus derive from BaseException
# instead of Exception. (KeyboardInterrupt is doing the same thing.)
class ServerShutdown(BaseException):
    pass


class SimpleWebInterface(BaseHTTPServer.BaseHTTPRequestHandler):
    server_version = 'SimpleWebInterface/2.0'
    servers = []
    serve_files = []
    serve_dirs = []

    pending_headers = None
    testing_user = None
    attempted_login = False

    current_cookies = {}
    passwords = {}

    log_file = sys.stderr

    def add_header(self, key, value):
        if self.pending_headers is None:
            self.pending_headers = []
        self.pending_headers.append((key, value))

    def clear_headers(self):
        self.pending_headers = None

    def get_user_from_id(self, id, pwd):
        valid = len(self.passwords) == 0
        if id in self.passwords and pwd == self.passwords[id]:
            valid = True

        if valid:
            while 1:
                value = ''.join([random.choice(string.ascii_lowercase)
                                 for i in range(20)])
                if value not in self.current_cookies:
                    break
            self.add_header('Set-Cookie', 'id=' + value)
            self.current_cookies[value] = id
            return id
        else:
            return None

    @classmethod
    def add_user(cls, id, pwd):
        cls.passwords[id] = pwd

    def get_user_from_cookie(self):
        if 'cookie' in self.headers:
            cookie = self.headers['cookie']
            for c in cookie.split(';'):
                if '=' in c:
                    name, val = c.split('=', 1)
                    name = name.strip()
                    val = val.strip()
                    if name == 'id' and val in self.current_cookies:
                        return self.current_cookies[val]
        return None

    def log_out(self):
        for k, v in self.current_cookies.items():
            if v == self.user:
                del self.current_cookies[k]
        self.add_header('Set-Cookie:', 'id=0')

    def default_login_form(self, target=None, **data):
        if self.attempted_login:
            message = 'Invalid password.  Try again.'
        else:
            message = 'Enter your user name and password:'
        data = ''.join(['<input type=hidden name=%s value=%s>' % kv
                        for kv in data.items() if kv[1] is not None])

        if target is None:
            target = self.currentArgs[0]

        # for some reason, using POST here doesn't work under IE6, but does
        #  under Firefox and Safari.
        method = 'GET'
        if 'User-Agent' in self.headers:
            agent = self.headers['User-Agent']
            if 'MSIE' not in agent:
                method = 'POST'

        return """<form action="%s" method=%s>%s<br>
        <label for=swi_id>Username: </label><input type=text name=swi_id><br>
        <label for=swi_pwd>Password: </label><input type=password name=swi_pwd>
        %s
        <input type=submit value="Log In">
        </form>""" % (target, method, message, data)

    def swi_favicon_ico(self):
        return ('image/ico', favicon)

    def do_POST(self):
        self.clear_headers()
        data = self.rfile.read(int(self.headers['Content-Length'])).decode('ascii')
        if 'multipart/form-data' in self.headers['Content-Type']:
            db = self.make_db_from_multipart(data)
        else:
            db = self.make_db_from_line(data)
        if '?' in self.path:
            self.path, data = self.path.split('?', 1)
            db2 = self.make_db_from_line(data)
            db.update(db2)

        if self.path == '':
            self.path = '/'
        self.path = self.fix_text(self.path)
        args = self.path[1:].split('/')
        self.handle_request(args, db)

    def do_GET(self):
        self.clear_headers()
        if self.path == '':
            self.path = '/'
        self.path = self.fix_text(self.path)

        db = {}
        if '?' in self.path:
            self.path, data = self.path.split('?', 1)
            db = self.make_db_from_line(data)

        args = self.path[1:].split('/')
        self.handle_request(args, db)

    def handle_ws_request(self, args, db):
        MAGIC = '258EAFA5-E914-47DA-95CA-C5AB0DC85B11'
        HSHAKE_RESP = ("HTTP/1.1 101 Switching Protocols\r\n"
                       "Upgrade: websocket\r\n"
                       "Connection: Upgrade\r\n"
                       "Sec-WebSocket-Accept: %s\r\n"
                       "\r\n")

        if args[0] == '':
            command = 'ws'
        else:
            command = 'ws_%s' % args[0]

        client = self.server.create_websocket(self.request)
        key = self.headers['Sec-WebSocket-Key'] + MAGIC
        key = key.encode('ascii')
        resp_data = (HSHAKE_RESP %
                     base64.b64encode(hashlib.sha1(key).digest()).decode('ascii'))
        client.socket.sendall(resp_data.encode('ascii'))
        client.set_blocking(False)

        self.user = self.get_user_from_cookie()
        if self.user is None and self.testing_user is not None:
            self.user = self.testing_user
        try:
            getattr(self, command)(*[client] + args[1:], **db)
        except socket.error as err:
            # Only print traceback if it isn't likely to be caused by the
            # server shutting down.
            if err.args[0] != errno.EPIPE and err.args[0] != errno.EBADF:
                traceback.print_exc()
        except Exception:
            traceback.print_exc()
        client.close()
        client.wait_for_close()

    def handle_request(self, args, db):
        self.currentArgs = args

        if 'Sec-WebSocket-Key' in self.headers:
            self.handle_ws_request(args, db)
            return

        if args[0] == '':
            command = 'swi'
        else:
            command = 'swi_%s' % args[0]
        command = command.replace('.', '_')

        if hasattr(self, command):
            ctype = 'text/html'
            self.user = self.get_user_from_cookie()
            if self.user is None and self.testing_user is not None:
                self.user = self.testing_user
            self.attempted_login = False
            if 'swi_id' in db and 'swi_pwd' in db:
                self.attempted_login = True
                self.user = self.get_user_from_id(db['swi_id'], db['swi_pwd'])
                del db['swi_id']
                del db['swi_pwd']
            elif len(self.passwords) == 0:
                self.user = self.get_user_from_id('', '')

            try:
                text = getattr(self, command)(*args[1:], **db)
            except:
                self.send_response(200)
                self.send_header('Content-type', 'text/html')
                self.end_headers()
                self.wfile.write("<html><body><pre>".encode("utf-8"))
                text = StringIO.StringIO()
                traceback.print_exc(file=text)
                text = text.getvalue()
                text = text.replace('<', '<')
                text = text.replace('>', '>')
                b = ("%s</pre></body></html>" % text).encode("utf-8")
                self.wfile.write(b)
                print(text)
            else:
                if isinstance(text, tuple):
                    ctype, text = text
                self.send_response(200)
                self.send_header('Content-type', ctype)
                if self.pending_headers is not None:
                    for k, v in self.pending_headers:
                        self.send_header(k, v)
                self.end_headers()
                if not isinstance(text, bytes):
                    text = text.encode("utf-8")
                self.wfile.write(text)
        elif self.path[1:] in self.serve_files:
            self.send_file(self.path[1:])
        elif self.path == '/robots.txt':
            self.send_response(200)
            self.send_header('Content-type', 'text/text')
            self.end_headers()
            self.wfile.write("User-agent: *\nDisallow: /".encode("utf-8"))
        else:
            for d in self.serve_dirs:
                if self.path[1:].startswith(d+'/'):
                    self.send_file(self.path[1:])
                    return
            self.send_response(200)
            self.send_header('Content-type', 'text/html')
            self.end_headers()
            self.wfile.write(("<html><body>Invalid request:"
                              "<pre>args=%s</pre><pre>db=%s</pre>"
                              "</body></html>" % (args, db)).encode("utf-8"))

    def send_file(self, path):
        self.send_response(200)
        type, enc = mimetypes.guess_type(path)
        self.send_header('Content-type', type)
        self.send_header('Content-encoding', enc)
        self.end_headers()
        self.wfile.write(file(path, 'rb').read())

    def make_db_from_multipart(self, data):
        db = {}
        boundary = data[:data.find('\n') + 1].strip()
        n = len(boundary)

        i = 0
        while True:
            j = data.find(boundary, i+n)
            if j < 0:
                break

            content = data[i + n + 2:j - 2]
            m = re.search(r'name="([^"]+)"', content)
            name = m.group(1)
            k = content.find('\r\n\r\n') + 4
            val = content[k:]

            m = re.search(r'filename="([^"]+)"', content[:k])
            if m is not None:
                filename = m.group(1)
                val = (filename, val)

            db[name] = val

            i = j
        return db

    def make_db_from_line(self, data):
        if '\n' in data:
            data, x = data.split('\n', 1)
            data = data.strip()
        db = {}
        for line in data.split('&'):
            if '=' in line:
                key, val = line.split('=', 1)
                val = self.fix_text(val)
                key = self.fix_text(key)
                if key in db:
                    v = db[key]
                    if not isinstance(v, list):
                        v = [v]
                    v.append(val)
                    val = v
                db[key] = val
        return db

    def fix_text(self, val):
        val = val.replace('+', ' ')
        i = 0
        while i < len(val) - 2:
            if val[i] == '%' and (val[i + 1] in '1234567890abcdefABCDEF' and
                                  val[i + 2] in '1234567890abcdefABCDEF'):
                c = chr(int(val[i + 1:i + 3], 16))
                val = val[:i] + c + val[i + 3:]
            i += 1
        return val

    def log_message(self, format, *args):
        if self.log_file is not None:
            self.log_file.write( "%s - - [%s] %s\n" % (
                    self.client_address[0], self.log_date_time_string(),
                    format % args))

    @classmethod
    def start(cls, viz, port=80, asynch=True, addr='', browser=False):
        server = cls.prepare_server(viz, port, asynch, addr, browser)
        cls.begin_lifecycle(server, asynch)

    @classmethod
    def prepare_server(cls, viz, port=80, asynch=True, addr='', browser=False):
        if asynch:
            server = AsyncHTTPServer((addr, port), cls)
        else:
            server = BaseHTTPServer.HTTPServer((addr, port), cls)
        server.viz = viz
        port = server.server_port
        if browser:
            cls.browser(port=port)

        cls.servers.append(server)
        return server

    @classmethod
    def _confirm_shutdown(cls, signum, frame):
        signal.signal(signal.SIGINT, cls._immediate_shutdown)
        sys.stdout.write("\nShut-down this web server (y/[n])? ")
        sys.stdout.flush()
        rlist, _, _ = select.select([sys.stdin], [], [], 10)
        if rlist:
            line = sys.stdin.readline()
            if line[0].lower() == 'y':
                raise ServerShutdown()
            else:
                print("Resuming...")
        else:
            print("No confirmation received. Resuming...")
        signal.signal(signal.SIGINT, cls._confirm_shutdown)

    @classmethod
    def _immediate_shutdown(cls, signum, frame):
        raise ServerShutdown()

    @classmethod
    def begin_lifecycle(cls, server, asynch=True, interactive=True):
        if interactive and not sys.platform.startswith('win'):
            signal.signal(signal.SIGINT, cls._confirm_shutdown)

        try:
            server.running = True
            server.serve_forever(poll_interval=0.02)
        except ServerShutdown:
            pass
        finally:
            server.running = False

            if interactive:
                print("Shutting down server...")

            ws_requests = []
            for ws in server.websockets:
                ws_requests.append(ws.socket)
                ws.close()

            # shut down any remaining threads
            # server.requests might be modified from other threads, so we need
            # a copy which we get in a thread-safe way be slicing it with [:].
            if asynch and server.requests is not None:
                for _, sock in server.requests:
                    if sock not in ws_requests:
                        try:
                            sock.shutdown(socket.SHUT_RDWR)
                            sock.close()
                        except socket.error:
                            pass

                for thread, _ in server.requests:
                    if thread.is_alive():
                        thread.join(0.05)

                n_zombie = sum(thread.is_alive()
                        for thread, _ in server.requests)
                if interactive and n_zombie > 0:
                    print("%d zombie threads will close abruptly" % n_zombie)

            cls.servers.remove(server)

    @classmethod
    def stop(cls):
        for server in cls.servers[:]:
            server.shutdown()

    @classmethod
    def browser(cls, port=80):
        threading.Thread(target=webbrowser.open,
                         args=('http://localhost:%d' % port,)).start()


class AsyncHTTPServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
    daemon_threads = True  # this ensures all spawned threads exit

    def __init__(self, *args, **kwargs):
        BaseHTTPServer.HTTPServer.__init__(self, *args, **kwargs)

        # keep track of open threads, so we can close them when we exit
        self._requests = []
        self.websockets = weakref.WeakSet()

    @property
    def requests(self):
        return self._requests[:]

    def create_websocket(self, request):
        client = ClientSocket(request, '')
        self.websockets.add(client)
        return client

    def process_request_thread(self, request, client_address):
        thread = threading.current_thread()
        self._requests.append((thread, request))
        SocketServer.ThreadingMixIn.process_request_thread(
            self, request, client_address)
        self._requests.remove((thread, request))

    def handle_error(self, request, client_address):
        exc_type, exc_value, _ = sys.exc_info()
        if exc_type is socket.error and (exc_value.args[0] == errno.EPIPE or
                exc_value.args[0] == errno.EBADF):
            return  # Probably caused by a server shutdown
        else:
            print(exc_type, exc_value, dir(exc_value))
            BaseHTTPServer.HTTPServer.handle_error(
                self, request, client_address)


favicon = ('\x00\x00\x01\x00\x01\x00\x10\x10\x00\x00\x01\x00\x18\x00h\x03\x00'
           '\x00\x16\x00\x00\x00(\x00\x00\x00\x10\x00\x00\x00 \x00\x00\x00\x01'
           '\x00\x18\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
           '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x009\x13\x115!\x0f/Z\x0f$'
           '\x81\x0e(\x9e\x10"\xbb &\xd7\x1c,\xe1\x1a9\xe2\x17a\xe3\x1au\xe4'
           '\x18\x93\xec!\xb5\xe6\x18\xcf\xe6\x18\xe7\xe8\'\xed\xdd62\x1c\x17'
           '\x185\x1e\x15W&\x18x+\x1f\x9e;\x1b\xbd@\x17\xd9F(\xe4?7\xe59X\xe99'
           'u\xe61\x8f\xe15\xaf\xd8)\xc6\xcf\x1a\xe1\xcd(\xe4\xc1;\'\x1f=\x173'
           ':\x16LG\x18oK"\x90U\x1c\xb6e\x1c\xdfy0\xe9kH\xeac`\xe0]v\xdeQ\x8a'
           '\xc89\x9b\xb3%\xb3\xb5\x1b\xca\xa8\x1f\xf5\x9c)0\x1eS#+P\x16Ed\x14'
           'br\x19\x90{\x1b\xbd\x93#\xe2\x8d7\xe3\x83K\xe3\x84e\xdbto\xc5Uq'
           '\xa7,\x8d\xa4 \xa3\x9b\x18\xc4\x92\x1a\xe1\x8c\x1e)\x13k\'$s%G\x83'
           '\x16Z\x8f(u\x9c \x9a\xa88\xca\xaaJ\xcf\x96`\xce\x8ci\xbcxm\xa7Nd'
           '\x8b)}\x81!\x8au\x19\xbbi"\xdei\x18\'\x14\x8b.\x1e\x8aA,\x89BM\x8b'
           'Dk\x98E~\x9eL\xa1\xab[\xaa\xa1^\xaf\x94`\xa6}[\x84J_j,f[\x1d\x83R'
           '\x1a\xa3L\x13\xcdJ\x18\x1d\x17\xa4? \xa3X!\x8e]/\x92XW\x97Sv'
           '\xa8^\x94\xabb\x99\xa0g\x9f\x94`\x89tSgJTQ2g@$\x7f8\x13\x9f8\x0b'
           '\xc7:\t\x13\x12\xb62\x1c\xb1d\x1a\x9a}$\x9cvK\xa2qm\xb5s\x8e\xb0m'
           '\x96\x9fw\x99\x98n}yf]YZFEr1:\x8a,\'\xaa.\x16\xde.\x10\x17\x1d'
           '\xce1\x1f\xd0^\x1c\xc1\x8b"\xae\x96@\xae\x8c[\xaf\x8d\x7f\xa8\x81'
           '\x8a\x94\x7f\x8d\x89}tqyZ]x7Qv\x1fK\x8e\x12>\xba\x1a&\xdd#\x1b'
           '\x15/\xda0-\xe0W#\xe2\x85,\xc9\x9fG\xbd\xa9g\xb9\xa4'
           '\x81\xa3\x93\x8e\x8b\x94\x93~\x8f\x80m\x95\\[\x94<T\x92%V\x9c'
           '\x0eI\xc0\x0bH\xdf\x1b9\x17D\xe1&E\xe8_>\xed\x9dN\xeb\xafb'
           '\xd5\xbc\x86\xcb\xaf\x94\xae\x99\xa4\x9a\x9d\xad\x8b\xad'
           '\x9ao\xc1{^\xbeW\\\xbeEZ\xb5\x15O\xdb\x14Q\xdd\x14e\x1a^\xdf0i'
           '\xddal\xe5\x97o\xf2\xb3\x85\xe8\xb6\x9e\xd4\xa8\xb4\xae\xa3\xbe'
           '\x96\xac\xc5\x89\xbd\xb9h\xd9\x90\\\xdfs[\xdae\\\xe0Ju\xd2 \x7f'
           '\xd4\x1b{!v\xde3}\xe7^\x8e\xdc\x80\x8c\xde\xa5\xaa\xe9\xad\xb5\xe4'
           '\xaa\xc9\xaa\xb2\xd6\x94\xbd\xdf\x81\xd7\xd1p\xdf\xabo\xdb\x8d'
           'p\xe8m{\xdeJ\x8a\xd61\x92\xce\x19\x9a\x1b\x95\xe9=\xa6\xe5W\xaa'
           '\xden\xb1\xde\x84\xc7\xe6\xa2\xca\xdc\xb2\xe0\xac\xbe\xe7\xa3\xcf'
           '\xe9\x9b\xe3\xd8\x94\xeb\xb0\x96\xe8\x97\x92\xe7}\x9a\xe7N\xa3'
           '\xd6$\xab\xcf\x11\xac\x1b\xc6\xe0&\xc9\xe9G\xd0\xeaj\xd0\xe2'
           'z\xda\xe0\x8a\xe1\xdd\xae\xe9\xcd\xc9\xeb\xc2\xe3\xe5\xc8\xf2'
           '\xd9\xdd\xe9\xbe\xcb\xeb\x9b\xbe\xe8n\xc6\xdeM\xcd\xd80\xd6\xd2'
           '\x14\xcd\x07\xe0\xdc"\xe8\xe4.\xeb\xeaJ\xea\xd8t\xe2\xd0\x85'
           '\xe7\xc9\x9a\xeb\xca\xc7\xed\xe1\xda\xe4\xeb\xec\xd3\xf5'
           '\xed\xbf\xee\xe4\x94\xed\xdcj\xe6\xd5Q\xeb\xd6>\xe5\xc5\x1b'
           '\xf0\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
           '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
           '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
           '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
           '\x00\x00')


class ClientSocket(object):
    def __init__(self, socket, addr):
        self.socket = socket
        self.addr = addr
        self.buffered_data = bytearray([])
        self.remote_close = False
        self._closing = False
        self._closed = False

    def set_timeout(self, timeout):
        self.socket.settimeout(timeout)

    def set_blocking(self, flag):
        self.socket.setblocking(flag)

    def read(self):
        # as a simple server, we expect to receive:
        #    - all data at one go and one frame
        #    - one frame at a time
        #    - text protocol
        #    - no ping pong messages
        # see: http://tools.ietf.org/html/rfc6455#section-5.2

        data = self.buffered_data
        try:
            data = data + bytearray(self.socket.recv(512))
        except socket.error as e:
            if e.errno == 11:  # no data available
                pass
            elif e.errno == 10035:  # no data available
                pass
            elif e.errno == 35:  # no data available
                pass
            elif e.errno == 9:  # "Bad file descriptor" means socket closed
                raise SocketClosedError("Cannot read from closed socket.")
            else:
                raise
        except socket.timeout:
            return None

        if len(data) == 0:
            return None

        if(len(data) < 6):
            raise Exception("Error reading data")

        fin = (data[0] >> 7) & 0x1
        rsv = (data[0] >> 4) & 0x07
        opcode = data[0] & 0x0F
        mask = (data[1] >> 7) & 0x01
        datalen = data[1] & 0x7F

        if opcode == 8:
            if not self._closing:
                self.close()  # acknowledge the close request
                self.remote_close = True;
            self._closed = True;
            try:
                self.socket.shutdown(socket.SHUT_RDWR)
                self.socket.close()
            except socket.error:
                pass
            raise SocketClosedError("Websocket has been closed")

        offset = 0

        if datalen == 126:
            datalen = 0
            for i in range(2):
                datalen = (datalen << 8) + data[2 + i]
            offset += 2
        elif datalen == 127:
            datalen = 0
            for i in range(8):
                datalen = (datalen << 8) + data[2 + i]
            offset += 8

        if (opcode != 1 and opcode != 10) or fin != 1 or mask != 1 or rsv != 0:
            print(dict(fin=fin, rsv=rsv, opcode=opcode, mask=mask,
                       datalen=datalen))
            return None

        str_data = ''
        if datalen > 0:
            stop_index = 6 + datalen + offset
            if len(data) < stop_index:
                self.buffered_data = data
                return None
            else:
                mask_key = data[2 + offset:6 + offset]
                masked_data = data[6 + offset : stop_index]
                unmasked_data = [masked_data[i] ^ mask_key[i % 4]
                                 for i in range(len(masked_data))]
                str_data = bytearray(unmasked_data).decode('ascii')

                self.buffered_data = data[stop_index:]

        if opcode == 10:
            return None  # pong response
        else:
            return str_data

    def write(self, data, binary=False, ping=False):
        if self._closing:
            raise SocketClosedError("Connection closing or closed.")

        if binary:
            code = 0b10000010
        elif ping:
            code = 0b10001001
        else:
            code = 0b10000001
            data += '\r\n'

        N = len(data)
        if N < 126:
            header = struct.pack('!BB', code, N)
        elif N <= 0xFFFF:
            header = struct.pack('!BBH', code, 126, N)
        else:
            header = struct.pack('!BBQ', code, 127, N)

        if not binary:
            data = data.encode('ascii')
        try:
            self._sendall(header)
            self._sendall(data)
        except socket.error as e:
            if e.errno == 32:  # Broken pipe
                raise SocketClosedError("Cannot write to socket.")
            else:
                raise

    def close(self):
        if not self._closing:
            self._closing = True
            code = 0b10001000
            packet = struct.pack('!BB', code, 0)
            self._sendall(packet)

    def wait_for_close(self):
        if not self._closed:
            try:
                while True:
                    self.read()
                    time.sleep(0.01)
            except SocketClosedError:
                return

    def _sendall(self, data):
        bytes_sent = 0
        while bytes_sent < len(data):
            bytes_sent += self.socket.send(data[bytes_sent:])


if __name__ == '__main__':
    class Demo(SimpleWebInterface):
        # http://localhost:8080/
        def swi(self):
            return 'Hello world!'

        # http://localhost:8080/test1?a=data
        # http://localhost:8080/test1/data
        def swi_test1(self, a='value'):
            return 'The value of a is "%s"' % a

        # http://localhost:8080/test2?a=data&b=data2
        # http://localhost:8080/test2/data/data2
        # http://localhost:8080/test2?b=data2
        def swi_test2(self, a='value', b='value'):
            return 'The value of a is "%s" and b is "%s"' % (a, b)

        # http://localhost:8080/page
        def swi_page(self):
            if self.user is None:
                return self.default_login_form()
            return '''
            Hello, %s.
            <p>Click <a href="otherPage">here</a> to go to another page.
            <p>Click <a href="logout">here</a> to log out.
            ''' % self.user

        # http://localhost:8080/otherPage
        def swi_otherPage(self):
            if self.user is None:
                return self.default_login_form()
            return '''
            You are now at the other page, %s.
            <p>Click <a href="page">here</a> to go back to the first page.
            <p>Click <a href="logout">here</a> to log out.
            ''' % self.user

        # http://localhost:8080/logout
        def swi_logout(self):
            self.log_out()
            return '''
            You are now logged out.
            <p>Click <a href="page">here</a> to the first page.
            <p>Click <a href="otherPage">here</a> to go to another page.
            '''

        # http://localhost:8080/timer
        def swi_timer(self):
            return """<html><head><script>
    function doInit() {
        var s;
        try {
            s = new WebSocket("ws://localhost:8080/timer");
            s.onopen = function(e) { console.log("connected");};
            s.onclose = function(e) { console.log("connection closed");};
            s.onerror = function(e) { console.log("connection error");};
            s.onmessage = function(e) { console.log("message: " + e.data);};
        } catch(ex) {
            console.log("connection exception: " + ex);
        }
    }
    </script></head>
    <body onload="doInit();">
    </body>
    </html>"""

        def ws_timer(self, client):
            import time
            while True:
                client.write(time.strftime('%H:%M:%S'))
                time.sleep(1)

    Demo.add_user('terry', 'password')
    Demo.start(port=8080, browser=True)



#!/usr/bin/env python

# This code is part of Grandalf
# Copyright (C) 2010-2012 Axel Tillequin (bdcht3@gmail.com)
# published under the GPLv2 license or EPLv1 license

#  Layouts are classes that provide graph drawing algorithms.
#
#  These classes all take a graph_core argument. The graph_core
#  topology will never be permanently modified by the drawing algorithm:
#  e.g. "dummy" node insertion, edge reversal for making the graph
#  acyclic and so on, are all kept inside the layout object.
#
from  numpy     import array,matrix
from .utils     import rand_ortho1,median_wh
from  sys       import getrecursionlimit,setrecursionlimit
from  bisect    import bisect
from  functools import cmp_to_key

try:
    xrange
except NameError:
    xrange = range

try:
    from itertools import izip
except:
    izip = zip

#  the VertexViewer class is responsible of providing
#  graphical attributes associated with a Vertex.
#  Instance of VertexViewer are generally created from
#  Vertex.data and then embedded in the 'view' field.
#  It is used by the layout algorithms the get vertex dimensions.
class  VertexViewer(object):
    def __init__(self,w=2,h=2,data=None):
        self.w = w
        self.h = h

    def __str__(self, *args, **kwargs):
        if hasattr(self, 'xy'):
            return 'VertexViewer (xy: %s) w: %s h: %s' % (self.xy, self.w, self.h)

        return 'VertexViewer (xy: None) w: %s h: %s' % (self.w, self.h)


#  SUGIYAMA LAYOUT
#------------------------------------------------------------------------------
#  The Sugiyama layout is the traditional "layered" graph layout
#  named 'dot' in graphviz.
#  This layout is quite efficient but heavily relies on drawing
#  heuristics. Adaptive drawing is limited to
#  extending the leaves only, but since the algorithm is quite fast
#  redrawing the entire graph (up to about a thousand nodes) gives
#  usually better results in less than a second.

class  _sugiyama_vertex_attr(object):
    def __init__(self,r=None,d=0):
        self.rank=r
        self.dummy=d
        self.pos=None
        self.x=0
        self.bar=None
    def __str__(self):
        s="(%3d,%3d) x=%s"%(self.rank,self.pos,str(self.x))
        if self.dummy: s="[d] %s"%s
        return s

class  DummyVertex(_sugiyama_vertex_attr):
    def __init__(self,r=None,viewclass=VertexViewer):
        self.view = viewclass()
        self.ctrl = None
        self.constrainer = False
        self.controlled = False
        _sugiyama_vertex_attr.__init__(self,r,d=1)
    def N(self,dir):
        assert dir==+1 or dir==-1
        return self.ctrl.get(self.rank+dir,[])
    def inner(self,dir):
        assert dir==+1 or dir==-1
        try:
            return self.ctrl[self.rank+dir][-1].dummy==1
        except KeyError:
            return False
        except AttributeError:
            return False
    def __str__(self):
        s="(%3d,%3d) x=%s"%(self.rank,self.pos,str(self.x))
        if self.dummy: s="[d] %s"%s
        if self.constrainer: s="[c] %s"%s
        return s

#------------------------------------------------------------------------------
# Layer is where Sugiyama layout organises nodes in hierarchical lists.
# The placement of nodes is done by the Sugiyama class, but it highly relies on
# the 'ordering' of nodes in each layer to reduce crossings.
# This ordering depends on the neighbors found in the upper or lower layers.
# WARNING: methods HIGHLY depend on layout.dirv state.
# TODO: see if grx dict can be stored inside Layers or needs to stay in layout.
#------------------------------------------------------------------------------
class Layer(list):
    __r    = None
    layout = None
    upper  = None
    lower  = None
    __x    = 1.
    ccount = None

    def __str__(self):
        s  = '<Layer %d'%self.__r
        s += ', len=%d'%len(self)
        xc = self.ccount or '?'
        s += ', crossings=%s>'%xc
        return s

    def setup(self,layout):
        self.layout = layout
        r = layout.layers.index(self)
        self.__r=r
        if len(self)>1: self.__x = 1./(len(self)-1)
        for i,v in enumerate(self):
            assert layout.grx[v].rank==r
            layout.grx[v].pos = i
            layout.grx[v].bar = i*self.__x
        if r>0:
            self.upper = layout.layers[r-1]
        if r<len(layout.layers)-1:
            self.lower = layout.layers[r+1]

    def nextlayer(self):
        return self.lower if self.layout.dirv==-1 else self.upper
    def prevlayer(self):
        return self.lower if self.layout.dirv==+1 else self.upper

    def order(self):
        sug = self.layout
        sug._edge_inverter()
        mvmt=[]
        c = self._cc()
        if c>0:
            for v in self:
                if sug.grx[v].dummy and v.constrainer:
                    mvmt.append(v)
                    continue
                bar = self._meanvalueattr(v,sug.order_attr)
                sug.grx[v].bar=bar
            while len(mvmt)>0:
                v = mvmt.pop()
                v.bar = sug.grx[v.ctrl[self.__r][0]].bar
            # now resort layers l according to bar value:
            self.sort(key=cmp_to_key(
                lambda x,y: ((sug.grx[x].bar > sug.grx[y].bar)
                             - (sug.grx[x].bar < sug.grx[y].bar))))
            # assign new position in layer l:
            for i,v in enumerate(self):
                if sug.grx[v].pos!=i: mvmt.append(v)
                sug.grx[v].pos = i
                #sug.grx[v].bar = i*self.__x
            # try count resulting crossings:
            c = self._ordering_reduce_crossings()
        self.layout._edge_inverter()
        self.ccount = c
        return mvmt

    # find new position of vertex v according to adjacency in prevlayer.
    # position is given by the mean value of adjacent positions.
    # experiments show that meanvalue heuristic performs better than median.
    def _meanvalueattr(self,v,att='bar'):
        sug = self.layout
        if sug.grx[v].dummy and v.constrainer:
            return v.ctrl[self.__r][0].bar
        if not self.prevlayer():
            return getattr(sug.grx[v],att)
        pos = [getattr(sug.grx[x],att) for x in self._neighbors(v)]
        if len(pos)==0:
            return getattr(sug.grx[v],att)
        return float(sum(pos))/len(pos)

    # find new position of vertex v according to adjacency in layer l+dir.
    # position is given by the median value of adjacent positions.
    # median heuristic is proven to achieve at most 3 times the minimum
    # of crossings (while barycenter achieve in theory the order of |V|)
    def _medianindex(self,v):
        assert self.prevlayer()!=None
        N = self._neighbors(v)
        g=self.layout.grx
        if g[v].dummy and v.controlled:
            for x in N:
                if g[x].dummy and x.constrainer:
                    return [g[x].pos]
        pos = [g[x].pos for x in N]
        lp = len(pos)
        if lp==0: return []
        pos.sort()
        pos = pos[::self.layout.dirh]
        i,j = divmod(lp-1,2)
        return [pos[i]] if j==0 else [pos[i],pos[i+j]]

    # neighbors refer to upper/lower adjacent nodes.
    # remember that v.N() provides neighbors of v within the graph, while
    # this method provides the Vertex and DummyVertex adjacent to v in the
    # upper or lower layer (depending on layout.dirv state).
    def _neighbors(self,v):
        assert self.layout.dag
        dirv = self.layout.dirv
        grxv=self.layout.grx[v]
        try: #(cache)
            return grxv.nvs[dirv]
        except AttributeError:
            grxv.nvs={-1:v.N(-1),+1:v.N(+1)}
            if grxv.dummy: return grxv.nvs[dirv]
            # v is real, v.N are graph neigbors but we need layers neighbors
            for d in (-1,+1):
                tr=grxv.rank+d
                for i,x in enumerate(v.N(d)):
                    if self.layout.grx[x].rank==tr:continue
                    e=v.e_with(x)
                    dum = self.layout.ctrls[e][tr][-1]
                    grxv.nvs[d][i]=dum
            return grxv.nvs[dirv]

    # counts (inefficently but at least accurately) the number of
    # crossing edges between layer l and l+dirv.
    # P[i][j] counts the number of crossings from j-th edge of vertex i.
    # The total count of crossings is the sum of flattened P:
    # x = sum(sum(P,[]))
    def _crossings(self):
        g=self.layout.grx
        P=[]
        for v in self:
            P.append([g[x].pos for x in self._neighbors(v)])
        for i,p in enumerate(P):
            candidates = sum(P[i+1:],[])
            for j,e in enumerate(p):
                p[j] = len(list(filter((lambda nx:nx<e), candidates)))
            del candidates
        return P

    # implementation of the efficient bilayer cross counting by insert-sort
    # (see Barth & Mutzel paper "Simple and Efficient Bilayer Cross Counting")
    def _cc(self):
        g=self.layout.grx
        P=[]
        for v in self:
            P.extend(sorted([g[x].pos for x in self._neighbors(v)]))
        # count inversions in P:
        s = []
        count = 0
        for i,p in enumerate(P):
            j = bisect(s,p)
            if j<i: count += (i-j)
            s.insert(j,p)
        return count

    def _ordering_reduce_crossings(self):
        assert self.layout.dag
        g = self.layout.grx
        N = len(self)
        X=0
        for i,j in izip(xrange(N-1),xrange(1,N)):
            vi = self[i]
            vj = self[j]
            ni = [g[v].pos for v in self._neighbors(vi)]
            Xij=Xji=0
            for nj in [g[v].pos for v in self._neighbors(vj)]:
                x = len(list(filter((lambda nx:nx>nj),ni)))
                Xij += x
                Xji += len(ni)-x
            if Xji<Xij:
                g[vi].pos,g[vj].pos = g[vj].pos,g[vi].pos
                self[i] = vj
                self[j] = vi
                X += Xji
            else:
                X += Xij
        return X

#------------------------------------------------------------------------------
#  The Sugiyama Layout Class takes as input a core_graph object and implements
#  an efficient drawing algorithm based on nodes dimensions provided through
#  a user-defined 'view' property in each vertex (see README.txt).
#------------------------------------------------------------------------------
class  SugiyamaLayout(object):
    def __init__(self,g):
        # drawing parameters:
        self.dirvh=0
        self.order_iter = 8
        self.order_attr = 'pos'
        self.xspace = 20
        self.yspace = 20
        self.dw = 10
        self.dh = 10
        # For layered graphs, vertices and edges need to have some additional
        # attributes that make sense only for this kind of layout:
        # update graph struct:
        self.g = g
        self.layers = []
        self.grx= {}
        self.ctrls = {}
        self.dag = False
        for v in self.g.V():
            assert hasattr(v,'view')
            self.grx[v] = _sugiyama_vertex_attr()
        self.dw,self.dh = median_wh([v.view for v in self.g.V()])
        self.dw = 8

    # initialize the layout engine based on required
    #  -list of edges for making the graph_core acyclic
    #  -list of root nodes.
    def init_all(self,roots=None,inverted_edges=None,cons=False,optimize=False):
        # For layered sugiyama algorithm, the input graph must be acyclic,
        # so we must provide a list of root nodes and a list of inverted edges.
        if roots==None:
            roots = list(filter(lambda x: len(list(x.e_in()))==0, self.g.sV))
        if inverted_edges==None:
            L = self.g.get_scs_with_feedback(roots)
            inverted_edges = list(filter(lambda x:x.feedback, self.g.sE))
        self.alt_e = inverted_edges
        # assign rank to all vertices:
        self.rank_all(roots,optimize)
        # add dummy vertex/edge for 'long' edges:
        self.ctrls['cons']=cons  # use "constrained edges" ?
        for e in self.g.E():
            self.setdummies(e,cons)
        # precompute some layers values:
        for l in self.layers: l.setup(self)

    # compute every node coordinates after converging to optimal ordering by N
    # rounds, and finally perform the edge routing.
    def draw(self,N=1.5):
        while N>0.5:
            for (l,mvmt) in self.ordering_step():
                pass
            N = N-1
        if N>0:
            for (l,mvmt) in self.ordering_step(oneway=True):
                pass
        self.setxy()
        self.draw_edges()

    def _edge_inverter(self):
        for e in self.alt_e:
            x,y = e.v
            e.v = (y,x)
        self.dag = not self.dag

    # internal state for alignment policy:
    # dirvh=0 -> dirh=+1, dirv=-1: leftmost upper
    # dirvh=1 -> dirh=-1, dirv=-1: rightmost upper
    # dirvh=2 -> dirh=+1, dirv=+1: leftmost lower
    # dirvh=3 -> dirh=-1, dirv=+1: rightmost lower
    @property
    def dirvh(self): return self.__dirvh
    @property
    def dirv(self): return self.__dirv
    @property
    def dirh(self): return self.__dirh
    @dirvh.setter
    def dirvh(self,dirvh):
        assert dirvh in range(4)
        self.__dirvh=dirvh
        self.__dirh,self.__dirv={0:(1,-1), 1:(-1,-1), 2:(1,1), 3:(-1,1)}[dirvh]
    @dirv.setter
    def dirv(self,dirv):
        assert dirv in (-1,+1)
        dirvh = (dirv+1)+(1-self.__dirh)/2
        self.dirvh = dirvh
    @dirh.setter
    def dirh(self,dirh):
        assert dirh in (-1,+1)
        dirvh = (self.__dirv+1)+(1-dirh)/2
        self.dirvh = dirvh

    # rank all vertices.
    # if list l is None, find initial rankable vertices (roots),
    # otherwise update ranking from these vertices.
    # The initial rank is based on precedence relationships,
    # optimal ranking may be derived from network flow (simplex).
    def rank_all(self,roots,optimize=False):
        self._edge_inverter()
        r = list(filter(lambda x: len(list(x.e_in()))==0 and x not in roots, self.g.sV))
        self._rank_init(list(roots)+list(r))
        if optimize: self._rank_optimize()
        self._edge_inverter()

    def _rank_init(self,unranked):
        assert self.dag
        scan = {}
        # set rank of unranked based on its in-edges vertices ranks:
        while len(unranked)>0:
            l = []
            for v in unranked:
                self.setrank(v)
                # mark out-edges has scan-able:
                for e in v.e_out(): scan[e]=True
                # check if out-vertices are rank-able:
                for x in v.N(+1):
                    if not (False in [scan.get(e,False) for e in x.e_in()]):
                        if x not in l: l.append(x)
            unranked=l

    # TODO: Network flow solver minimizing total edge length
    # Also interesting: http://jgaa.info/accepted/2005/EiglspergerSiebenhallerKaufmann2005.9.3.pdf
    def _rank_optimize(self):
        assert self.dag
        for l in reversed(self.layers):
            for v in l:
                gv = self.grx[v]
                for x in v.N(-1):
                   if all((self.grx[y].rank>=gv.rank for y in x.N(+1))):
                        gx = self.grx[x]
                        self.layers[gx.rank].remove(x)
                        gx.rank = gv.rank-1
                        self.layers[gv.rank-1].append(x)


    def setrank(self,v):
        assert self.dag
        r=max([self.grx[x].rank for x in v.N(-1)]+[-1])+1
        self.grx[v].rank=r
        # add it to its layer:
        try:
            self.layers[r].append(v)
        except IndexError:
            assert r==len(self.layers)
            self.layers.append(Layer([v]))

    def dummyctrl(self,r,ctrl):
        dv = DummyVertex(r)
        dv.view.w,dv.view.h=self.dw,self.dh
        self.grx[dv] = dv
        dv.ctrl = ctrl
        try:
            ctrl[r].append(dv)
        except KeyError:
            ctrl[r] = [dv]
        self.layers[r].append(dv)
        return dv

    def setdummies(self,e,with_constraint=True):
        v0,v1 = e.v
        r0,r1 = self.grx[v0].rank,self.grx[v1].rank
        if r0>r1:
            assert e in self.alt_e
            v0,v1 = v1,v0
            r0,r1 = r1,r0
        elif r0==r1:
            raise ValueError('bad ranking')
        spanover=xrange(r0+1,r1)
        if (r1-r0)>1:
            # "dummy vertices" are stored in the edge ctrl dict,
            # keyed by their rank in layers.
            ctrl=self.ctrls[e]={}
            ctrl[r0]=[v0]
            ctrl[r1]=[v1]
            for r in spanover:
                self.dummyctrl(r,ctrl)
            if e in self.alt_e and with_constraint:
                dv0 = self.dummyctrl(r0,ctrl)
                dv1 = self.dummyctrl(r1,ctrl)
                dv0.constrainer=True
                dv1.constrainer=True
                ctrl[r0+1][0].controlled=True
                ctrl[r1-1][0].controlled=True

    # iterator that computes all node coordinates and edge routing after
    # just one step (one layer after the other from top to bottom to top).
    # Purely inefficient ! Use it only for "animation" or debugging purpose.
    def draw_step(self):
        ostep = self.ordering_step()
        for s in ostep:
            self.setxy()
            self.draw_edges()
            yield s

    def ordering_step(self,oneway=False):
        self.dirv=-1
        crossings = 0
        for l in self.layers:
            mvmt = l.order()
            crossings += l.ccount
            yield (l,mvmt)
        if oneway or (crossings == 0):
            return
        self.dirv=+1
        while l:
            mvmt = l.order()
            yield (l,mvmt)
            l = l.nextlayer()

    # algorithm by Brandes & Kopf:
    def setxy(self):
        self._edge_inverter()
        self._detect_alignment_conflicts()
        inf = float('infinity')
        # initialize vertex coordinates attributes:
        for l in self.layers:
            for v in l:
                self.grx[v].root  = v
                self.grx[v].align = v
                self.grx[v].sink  = v
                self.grx[v].shift = inf
                self.grx[v].X     = None
                self.grx[v].x     = [0.0]*4
        curvh = self.dirvh # save current dirvh value
        for dirvh in xrange(4):
            self.dirvh = dirvh
            self._coord_vertical_alignment()
            self._coord_horizontal_compact()
        self.dirvh = curvh # restore it
        # vertical coordinate assigment of all nodes:
        Y = 0
        for l in self.layers:
            dY = max([v.view.h/2. for v in l])
            for v in l:
                vx = sorted(self.grx[v].x)
                # mean of the 2 medians out of the 4 x-coord computed above:
                avgm = (vx[1]+vx[2])/2.
                # final xy-coordinates :
                v.view.xy = (avgm,Y+dY)
            Y += 2*dY+self.yspace
        self._edge_inverter()

    # mark conflicts between edges:
    # inner edges are edges between dummy nodes
    # type 0 is regular crossing regular (or sharing vertex)
    # type 1 is inner crossing regular (targeted crossings)
    # type 2 is inner crossing inner (avoided by reduce_crossings phase)
    def _detect_alignment_conflicts(self):
        curvh = self.dirvh # save current dirvh value
        self.dirvh=0
        self.conflicts = []
        for L in self.layers:
            last = len(L)-1
            prev = L.prevlayer()
            if not prev: continue
            k0=0
            k1_init=len(prev)-1
            l=0
            for l1,v in enumerate(L):
                if not self.grx[v].dummy: continue
                if l1==last or v.inner(-1):
                    k1=k1_init
                    if v.inner(-1):
                        k1=self.grx[v.N(-1)[-1]].pos
                    for vl in L[l:l1+1]:
                        for vk in L._neighbors(vl):
                            k = self.grx[vk].pos
                            if (k<k0 or k>k1):
                                self.conflicts.append((vk,vl))
                    l=l1+1
                    k0=k1
        self.dirvh = curvh # restore it

    # vertical alignment highly depends on dirh/dirv state.
    def _coord_vertical_alignment(self):
        dirh,dirv = self.dirh,self.dirv
        g = self.grx
        for l in self.layers[::-dirv]:
            if not l.prevlayer(): continue
            r=None
            for vk in l[::dirh]:
                for m in l._medianindex(vk):
                    # take the median node in dirv layer:
                    um = l.prevlayer()[m]
                    if g[um].dummy and um.controlled and not g[vk].dummy:
                        continue
                    # if vk is "free" align it with um's root
                    if g[vk].align == vk:
                        if dirv==1: vpair = (vk,um)
                        else:       vpair = (um,vk)
                        # if vk<->um link is used for alignment
                        if (vpair not in self.conflicts) and \
                           (r==None or dirh*r<dirh*m):
                            g[um].align = vk
                            g[vk].root = g[um].root
                            g[vk].align = g[vk].root
                            r = m


    def _coord_horizontal_compact(self):
        limit=getrecursionlimit()
        N=len(self.layers)+10
        if N>limit:
            setrecursionlimit(N)
        dirh,dirv = self.dirh,self.dirv
        g = self.grx
        L = self.layers[::-dirv]
        # recursive placement of blocks:
        for l in L:
            for v in l[::dirh]:
                if g[v].root==v:
                    self.__place_block(v)
        setrecursionlimit(limit)
        # mirror all nodes if right-aligned:
        if dirh==-1:
            for l in L:
                for v in l:
                    x = g[v].X
                    if x: g[v].X = -x
        # then assign x-coord of its root:
        inf=float('infinity')
        rb=inf
        for l in L:
            for v in l[::dirh]:
                g[v].x[self.dirvh] = g[g[v].root].X
                rs = g[g[v].root].sink
                s = g[rs].shift
                if s<inf:
                    g[v].x[self.dirvh] += dirh*s
                rb = min(rb,g[v].x[self.dirvh])
        # normalize to 0, and reinit root/align/sink/shift/X
        for l in self.layers:
            for v in l:
                #g[v].x[dirvh] -= rb
                g[v].root = g[v].align = g[v].sink = v
                g[v].shift = inf
                g[v].X = None

    def __place_block(self,v):
        g = self.grx
        if g[v].X==None:
            # every block is initially placed at x=0
            g[v].X = 0.0
            # place block in which v belongs:
            w = v
            while 1:
                j = g[w].pos-self.dirh # predecessor in rank must be placed
                r = g[w].rank
                if 0<= j <len(self.layers[r]):
                    wprec = self.layers[r][j]
                    delta = self.xspace+(wprec.view.w + w.view.w)/2.   # abs positive minimum displ.
                    # take root and place block:
                    u = g[wprec].root
                    self.__place_block(u)
                    # set sink as sink of prec-block root
                    if g[v].sink==v:
                        g[v].sink = g[u].sink
                    if g[v].sink!=g[u].sink:
                        s = g[u].sink
                        newshift = g[v].X-(g[u].X+delta)
                        g[s].shift = min(g[s].shift,newshift)
                    else:
                        g[v].X = max(g[v].X,(g[u].X+delta))
                # take next node to align in block:
                w = g[w].align
                # quit if self aligned
                if w==v: break

    # Basic edge routing applied only for edges with dummy points.
    # Enhanced edge routing can be performed by using the apropriate
    # route_with_ functions from routing.py in the edges' view.
    def draw_edges(self):
        for e in self.g.E():
            if hasattr(e,'view'):
                l=[]
                r0,r1 = None,None
                if self.ctrls.has_key(e):
                    D = self.ctrls[e]
                    r0,r1 = self.grx[e.v[0]].rank,self.grx[e.v[1]].rank
                    if r0<r1:
                        ranks = xrange(r0+1,r1)
                    else:
                        if self.ctrls['cons']:
                            ranks = xrange(r0,r1-1,-1)
                        else:
                            ranks = xrange(r0-1,r1,-1)
                    l = [D[r][-1].view.xy for r in ranks]
                l.insert(0,e.v[0].view.xy)
                l.append(e.v[1].view.xy)
                if self.ctrls['cons'] and r0>r1:
                    dy = e.v[0].view.h/2. + self.yspace/3.
                    x,y = zip(l[0],l[1])
                    y = max(y)+dy
                    l.insert(1,(x[0],y))
                    l.insert(2,(x[1],y))
                    dy = e.v[1].view.h/2. + self.yspace/3.
                    x,y = zip(l[-1],l[-2])
                    y = min(y)-dy
                    l.insert(-1,(x[0],y))
                    l.insert(-2,(x[1],y))
                try:
                    self.route_edge(e,l)
                except AttributeError:
                    pass
                e.view.setpath(l)


#  DIRECTED GRAPH WITH CONSTRAINTS LAYOUT
#------------------------------------------------------------------------------
class  DigcoLayout(object):
    def __init__(self,g):
        # drawing parameters:
        self.xspace = 10
        self.yspace = 10
        self.dr     = 10
        self.debug=False

        self.g = g
        self.levels = []
        for i,v in enumerate(self.g.V()):
            assert hasattr(v,'view')
            v.i = i
            self.dr = max((self.dr,v.view.w,v.view.h))
        # solver parameters:
        self._cg_max_iter = g.order()
        self._cg_tolerance = 1.0e-6
        self._eps = 1.0e-5
        self._cv_max_iter = self._cg_max_iter

    def init_all(self,alpha=0.1,beta=0.01):
        y = None
        if self.g.directed:
            # partition g in hierarchical levels:
            y = self.part_to_levels(alpha,beta)
        # initiate positions (y and random in x):
        self.Z = self._xyinit(y)

    def draw(self,N=None):
        if N is None: N = self._cv_max_iter
        self.Z = self._optimize(self.Z,limit=N)
        # set view xy from near-optimal coords matrix:
        for v in self.g.V():
            v.view.xy = (self.Z[v.i][0,0]*self.dr,
                         self.Z[v.i][0,1]*self.dr)
        self.draw_edges()

    def draw_step(self):
        for x in xrange(self._cv_max_iter):
            self.draw(N=1)
            self.draw_edges()
            yield

    # Basic edge routing with segments
    def draw_edges(self):
        for e in self.g.E():
            if hasattr(e,'view'):
                l=[e.v[0].view.xy,e.v[1].view.xy]
                try:
                    self.route_edge(e,l)
                except AttributeError:
                    pass
                e.view.setpath(l)

    # partition the nodes into levels:
    def part_to_levels(self,alpha,beta):
        opty,err = self.optimal_arrangement()
        ordering = zip(opty,self.g.sV)
        eps = alpha*(opty.max()-opty.min())/(len(opty)-1)
        eps = max(beta,eps)
        ordering.sort(reverse=True)
        l = []
        self.levels.append(l)
        for i in xrange(len(ordering)-1):
            y,v = ordering[i]
            l.append(v)
            v.level = self.levels.index(l)
            if (y-ordering[i+1][0])>eps:
                l=[]
                self.levels.append(l)
        y,v = ordering[-1]
        l.append(v)
        v.level = self.levels.index(l)
        return opty

    def optimal_arrangement(self):
        b = self.balance()
        y = rand_ortho1(self.g.order())
        return self._conjugate_gradient_L(y,b)

    # balance vector is assembled in finite-element way...
    # this is faster than computing b[i] for each i.
    def balance(self):
        b = array([0.0]*self.g.order(),dtype=float)
        for e in self.g.E():
            s = e.v[0]
            d = e.v[1]
            q = e.w*(self.yspace+(s.view.h+d.view.h)/2.)
            b[s.i] += q
            b[d.i] -= q
        return b

    # We compute the solution Y of L.Y = b by conjugate gradient method
    # (L is semi-definite positive so Y is unique and convergence is O(n))
    # note that only arrays are involved here...
    def _conjugate_gradient_L(self,y,b):
        Lii = self.__Lii_()
        r = b - self.__L_pk(Lii,y)
        p = array(r,copy=True)
        rr = sum(r*r)
        for k in xrange(self._cg_max_iter):
            try:
                Lp = self.__L_pk(Lii,p)
                alpha = rr/sum(p*Lp)
                y += alpha/p
                r -= alpha*Lp
                newrr = sum(r*r)
                beta = newrr/rr
                rr = newrr
                if rr<self._cg_tolerance: break
                p = r + beta*p
            except ZeroDivisionError:
                return (None,rr)
        return (y,rr)

    # _xyinit can use diagonally scaled initial vertices positioning to provide
    # better convergence in constrained stress majorization
    def _xyinit(self,y=None):
        if y is None:
            y = rand_ortho1(self.g.order())
        x = rand_ortho1(self.g.order())
        # translate and normalize:
        x = x-x[0]
        y = y-y[0]
        sfactor = 1.0/max(y.max(),x.max())
        return matrix(zip(x*sfactor,y*sfactor))

    # provide the diagonal of the Laplacian matrix of g
    # the rest of L (sparse!) is already stored in every edges.
    def __Lii_(self):
        Lii = []
        for v in self.g.V():
            Lii.append(sum([e.w for e in v.e]))
        return array(Lii,dtype=float)

    # we don't compute the L.Pk matrix/vector product here since
    # L is sparse (order of |E| not |V|^2 !) so we let each edge
    # contribute to the resulting L.Pk vector in a FE assembly way...
    def __L_pk(self,Lii,pk):
        y = Lii*pk
        for e in self.g.sE:
            i1 = e.v[0].i
            i2 = e.v[1].i
            y[i1] -= e.w*pk[i2]
            y[i2] -= e.w*pk[i1]
        return y

    # conjugate_gradient with given matrix Lw:
    # it is assumed that b is not a multivector,
    # so _cg_Lw should be called in all directions separately.
    # note that everything is a matrix here, (arrays are row vectors only)
    def _cg_Lw(self,Lw,z,b):
        scal = lambda U,V: float(U.transpose()*V)
        r = b - Lw*z
        p = matrix(r,copy=True)
        rr = scal(r,r)
        for k in xrange(self._cg_max_iter):
            if rr<self._cg_tolerance: break
            Lp = Lw*p
            alpha = rr/scal(p,Lp)
            z = z + alpha*p
            r = r - alpha*Lp
            newrr = scal(r,r)
            beta = newrr/rr
            rr = newrr
            p  = r + beta*p
        return (z,rr)

    def __Dij_(self):
        Dji = []
        for v in self.g.V():
            wd = self.g.dijkstra(v)
            Di = [wd[w] for w in self.g.V()]
            Dji.append(Di)
        # at this point  D is stored by rows,
        # but anymway it's a symmetric matrix
        return matrix(Dji,dtype=float)

    # returns matrix -L^w
    def __Lij_w_(self):
        self.Dij = self.__Dij_()  # we keep D also for L^Z computations
        Lij = self.Dij.copy()
        n = self.g.order()
        for i in xrange(n):
            d = 0
            for j in xrange(n):
                if j==i: continue
                Lij[i,j] = 1.0/self.Dij[i,j]**2
                d += Lij[i,j]
            Lij[i,i] = -d
        return Lij

    # returns vector -L^Z.Z:
    def __Lij_Z_Z(self,Z):
        from math import sqrt
        scal = lambda U,V: float(U.transpose()*V)
        def dist(Zi,Zk):
            v = (Zi-Zk).transpose()
            return sqrt(scal(v,v))
        n = self.g.order()
        # init:
        lzz = Z.copy()*0.0 # lzz has dim Z (n x 2)
        liz = matrix([0.0]*n) # liz is a row of L^Z (size n)
        # compute lzz = L^Z.Z while assembling L^Z by row (liz):
        for i in xrange(n):
            iterk_except_i = (k for k in xrange(n) if k!=i)
            for k in iterk_except_i:
                liz[0,k] = 1.0/(self.Dij[i,k]*dist(Z[i],Z[k]))
            liz[0,i] = 0.0 # forced, otherwise next liz.sum() is wrong !
            liz[0,i] = -liz.sum()
            # now that we have the i-th row of L^Z, just dotprod with Z:
            lzz[i] = liz*Z
        return lzz

    def _optimize(self,Z,limit=100):
        scal = lambda U,V: float(U.transpose()*V)
        Lw = self.__Lij_w_()
        K = self.g.order()*(self.g.order()-1.0)/2.0
        stress = float('inf')
        count=0
        deep=0
        b  = self.__Lij_Z_Z(Z)
        while count<limit:
            if self.debug:
                print("count %d"%count)
                print("Z = ",Z)
                print("b = ",b)
            # find next Z by solving Lw.Z = b in every direction:
            x,xerr = self._cg_Lw(Lw[1:,1:],Z[1:,0],b[1:,0])
            y,yerr = self._cg_Lw(Lw[1:,1:],Z[1:,1],b[1:,1])
            Z[1:,0] = x
            Z[1:,1] = y
            if self.debug:
                print(" cg -> ")
                print(Z,xerr,yerr)
            # compute new stress:
            FZ = K-float(x.transpose()*b[1:,0] + y.transpose()*b[1:,1])
            # precompute new b:
            b  = self.__Lij_Z_Z(Z)
            # update new stress:
            FZ += 2*float(x.transpose()*b[1:,0] + y.transpose()*b[1:,1])
            # test convergence:
            print('stress=%.10f'%FZ)
            if stress==0.0 : break
            elif abs((stress-FZ)/stress)<self._eps:
                if deep==2:
                    break
                else:
                    deep += 1
            stress=FZ
            count += 1
        return Z


#------------------------------------------------------------------------------
class  DwyerLayout(object):
    def __init__(self):
        raise NotImplementedError



from .slider import Slider, SliderTemplate
from .value import Value, ValueTemplate
from .xyvalue import XYValue, XYValueTemplate
from .sim_control import SimControl, SimControlTemplate
from .raster import Raster, RasterTemplate
from .voltage import Voltage, VoltageTemplate
from .pointer import Pointer, PointerTemplate
from .netgraph import NetGraph, NetGraphTemplate
from .ace_editor import AceEditor, AceEditorTemplate



#!/usr/bin/env python
import imp
import io
import os

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()

from setuptools import find_packages, setup


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)

root = os.path.dirname(os.path.realpath(__file__))
version_module = imp.load_source(
    'version', os.path.join(root, 'nengo_gui', 'version.py'))
description = "Web-based GUI for building and visualizing Nengo models."
long_description = read('README.rst', 'CHANGES.rst')

setup(
    name="nengo_gui",
    version=version_module.version,
    author="CNRGlab at UWaterloo",
    author_email="celiasmith@uwaterloo.ca",
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'nengo_gui = nengo_gui:main',
        ]
    },
    scripts=[],
    url="https://github.com/nengo/nengo_gui",
    license="https://github.com/nengo/nengo_gui/blob/master/LICENSE.md",
    description=description,
    install_requires=[
        "nengo",
    ],
)



# Nengo Example: Combining
#
# This example demonstrates how to create a neuronal ensemble that will
# combine two 1-D inputs into one 2-D representation.

import nengo
import numpy as np

model = nengo.Network()
with model:
    # Our input ensembles consist of 100 leaky integrate-and-fire neurons,
    # representing a one-dimensional signal
    a = nengo.Ensemble(n_neurons=100, dimensions=1)
    b = nengo.Ensemble(n_neurons=100, dimensions=1)

    # The output ensemble consists of 200 leaky integrate-and-fire neurons,
    # representing a two-dimensional signal
    output = nengo.Ensemble(200, dimensions=2)

    # Create input nodes generating the sine and cosine
    sin = nengo.Node(output=np.sin)
    cos = nengo.Node(output=np.cos)

    nengo.Connection(sin, a)
    nengo.Connection(cos, b)

    # The square brackets define which dimension the input will project to
    nengo.Connection(a, output[1])
    nengo.Connection(b, output[0])


import numpy as np

import nengo

D = 9
cols = int(np.sqrt(D))
size = 150

model = nengo.Network()
with model:
    for i in range(D):
        def waves(t, i=i):
            return np.sin(t + np.arange(i + 1) * 2 * np.pi / (i + 1))
        node = nengo.Node(waves)

if __name__ == '__main__':
    import nengo_gui
    nengo_gui.Viz(__file__).start()



import json

import nengo

from nengo_gui.components.component import Component, Template
import nengo_gui.monkey

class AceEditor(Component):
    def __init__(self, viz, config, uid):
        super(AceEditor, self).__init__(viz, config, uid)
        self.viz = viz
        self.uid = uid
        if self.viz.viz.interactive:
            self.current_code = self.viz.viz.code
            self.serve_code = True
            self.last_error = None
            self.last_stdout = None

    def update_client(self, client):
        if not self.viz.viz.interactive:
            return
        if self.serve_code:
            i = json.dumps({'code': self.current_code})
            client.write(i)
            self.serve_code = False
        if nengo_gui.monkey.is_executing():
            return
        error = self.viz.current_error
        stdout = nengo_gui.monkey.patch.stdout.getvalue()
        if error != self.last_error or stdout != self.last_stdout:
            if error is None:
                short_msg = None
            else:
                short_msg = error['trace'].rsplit('\n', 2)[-2]
            client.write(json.dumps({'error': error,
                                     'short_msg':short_msg,
                                     'stdout':stdout}))
            self.last_error = error
            self.last_stdout = stdout

    def javascript(self):
        args = json.dumps(dict(active=self.viz.viz.interactive))
        return 'ace_editor = new Nengo.Ace("%s", %s)' % (self.uid, args)

    def message(self, msg):
        if not self.viz.viz.interactive:
            return
        data = json.loads(msg)
        self.current_code = data['code']

        if data['save']:
            try:
                with open(self.viz.viz.filename, 'w') as f:
                    f.write(self.current_code)
            except IOError:
                print("Could not save %s; permission denied" %
                      self.viz.viz.filename)
                self.viz.new_code = self.current_code
        else:
            self.viz.new_code = self.current_code


class AceEditorTemplate(Template):
    cls = AceEditor
    config_params = {}



# Nengo Example: Squaring the Input

# This demo shows you how to construct a network that squares the value
# encoded in a first population in the output of a second population.

# Create the model object
import nengo
model = nengo.Network()
with model:
    # Create two ensembles of 100 leaky-integrate-and-fire neurons
    a = nengo.Ensemble(n_neurons=100, dimensions=1)
    b = nengo.Ensemble(n_neurons=100, dimensions=1)

    # Create an input node
    stim = nengo.Node([0])

    # Connect the input node to ensemble A
    nengo.Connection(stim, a)

    # Define the squaring function
    def square(x):
        return x[0] * x[0]

    # Connection ensemble a to ensemble b
    nengo.Connection(a, b, function=square)


import time
import struct
import traceback

import numpy as np
import nengo
import os
import os.path
import json

from nengo_gui.components.component import Component, Template
import nengo_gui.monkey

class SimControl(Component):
    def __init__(self, viz, config, uid, dt=0.001):
        super(SimControl, self).__init__(viz, config, uid)
        self.viz = viz
        self.paused = True
        self.last_tick = None
        self.rate = 0.0
        self.model_dt = dt
        self.rate_tau = 1.0
        self.last_send_rate = None
        self.shown_time = config.shown_time
        self.kept_time = config.kept_time
        self.sim_ticks = 0
        self.skipped = 1
        self.time = 0.0
        self.last_status = None
        self.next_ping_time = None
        self.send_config_options = False

    def add_nengo_objects(self, viz):
        with viz.model:
            self.node = nengo.Node(self.control, size_out=0)

    def remove_nengo_objects(self, viz):
        viz.model.nodes.remove(self.node)

    def finish(self):
        self.viz.finish()

    def control(self, t):
        self.time = t
        self.sim_ticks += 1

        now = time.time()
        if self.last_tick is not None:
            dt = now - self.last_tick
            if dt == 0:
                self.skipped += 1
            else:
                rate = self.model_dt * self.skipped / dt
                decay = np.exp(-dt / self.rate_tau)
                self.rate *= decay
                self.rate += (1 - decay) * rate
                self.skipped = 1

        self.last_tick = now

        while self.paused and self.viz.sim is not None:
            time.sleep(0.01)
            self.last_tick = None

    def config_settings(self, data):
        for i in data:
            print(i)

    def update_client(self, client):
        now = time.time()
        # send off a ping now and then so we'll notice when connection closes
        if self.next_ping_time is None or now > self.next_ping_time:
            client.write('', ping=True)
            self.next_ping_time = now + 2.0

        if self.viz.changed:
            self.paused = True
            self.viz.sim = None
            self.viz.changed = False
        if not self.paused:
            client.write(struct.pack('<ff', self.time, self.rate), binary=True)
        status = self.get_status()
        if status != self.last_status:
            client.write('status:%s' % status)
            self.last_status = status
        if self.send_config_options == True:
            client.write('sims:' + self.backend_options_html())
            client.write('config' +
                'Nengo.Toolbar.prototype.config_modal_show();')
            self.send_config_options = False

    def get_status(self):
        if self.paused:
            return 'paused'
        elif self.viz.sim is None:
            return 'building'
        else:
            return 'running'

    def javascript(self):
        info = dict(uid=self.uid)
        fn = json.dumps(self.viz.viz.filename)
        js = self.javascript_config(info)
        return ('sim = new Nengo.SimControl(control, %s);\n'
                'toolbar = new Nengo.Toolbar(%s); ' % (js, fn))

    def message(self, msg):
        if msg == 'pause':
            self.paused = True
        elif msg == 'config':
            self.send_config_options = True
        elif msg == 'continue':
            if self.viz.sim is None:
                self.viz.rebuild = True
            self.paused = False
        elif msg == 'reset':
            self.viz.sim = None
        elif msg[:8] == 'backend:':
            self.viz.backend = msg[8:]
            self.viz.changed = True

    def backend_options_html(self):
        items = []
        for module in nengo_gui.monkey.found_modules:
            if module == self.viz.backend:
                selected = ' selected'
            else:
                selected = ''
            item = '<option %s>%s</option>' % (selected, module)
            items.append(item)
        return ''.join(items)


class SimControlTemplate(Template):
    cls = SimControl
    config_params = dict(shown_time=0.5, kept_time=4.0)
